# CHIPSEC: Platform Security Assessment Framework
# Copyright (c) 2010-2021, Intel Corporation
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; Version 2.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
#
# Contact information:
# chipsec@intel.com
#

"""
Main UEFI component using platform specific and common UEFI functionality
"""

import struct
import os
from typing import Dict, List, Optional, Tuple, TYPE_CHECKING
if TYPE_CHECKING:
    from chipsec.hal.uefi_common import S3BOOTSCRIPT_ENTRY, EFI_SYSTEM_TABLE
    from chipsec.hal.uefi_platform import EfiVariableType, EfiTableType
from chipsec.hal import hal_base, uefi_platform
from chipsec.hal.spi import SPI, BIOS
from chipsec.hal.uefi_common import EFI_VENDOR_TABLE, EFI_VENDOR_TABLE_SIZE, EFI_VENDOR_TABLE_FORMAT, EFI_TABLE_HEADER_SIZE, EFI_TABLE_HEADER, EFI_TABLES, MAX_EFI_TABLE_SIZE
from chipsec.hal.uefi_common import S3BootScriptOpcode, S3_BOOTSCRIPT_VARIABLES, parse_efivar_file, EFI_REVISIONS, AUTH_SIG_VAR, ESAL_SIG_VAR
from chipsec.hal.uefi_common import EFI_VARIABLE_AUTHENTICATED_WRITE_ACCESS, EFI_VARIABLE_TIME_BASED_AUTHENTICATED_WRITE_ACCESS, EFI_VARIABLE_APPEND_WRITE, EFI_VARIABLE_NON_VOLATILE
from chipsec.hal.uefi_common import EFI_VARIABLE_BOOTSERVICE_ACCESS, EFI_VARIABLE_RUNTIME_ACCESS, EFI_VARIABLE_HARDWARE_ERROR_RECORD, SECURE_BOOT_SIG_VAR
from chipsec.hal.uefi_common import IS_VARIABLE_ATTRIBUTE, EFI_TABLE_HEADER_FMT, EFI_SYSTEM_TABLE_SIGNATURE, EFI_RUNTIME_SERVICES_SIGNATURE, EFI_BOOT_SERVICES_SIGNATURE
from chipsec.hal.uefi_common import EFI_DXE_SERVICES_TABLE_SIGNATURE, EFI_CONFIGURATION_TABLE, ACPI_VARIABLE_SET_STRUCT_SIZE
from chipsec.library.logger import logger, print_buffer_bytes
from chipsec.library.file import write_file, read_file
from chipsec.library.defines import bytestostring
from chipsec.helper.oshelper import OsHelperError

########################################################################################################
#
# S3 Resume Boot-Script Parsing Functionality
#
########################################################################################################


def parse_script(script: bytes, log_script: bool = False) -> List['S3BOOTSCRIPT_ENTRY']:
    off = 0
    entry_type = 0
    s3_boot_script_entries = []
    len_s = len(script)

    if log_script:
        logger().log('[uefi] +++ S3 Resume Boot-Script +++\n')
    script_type, script_header_length = uefi_platform.id_s3bootscript_type(script, log_script)
    off += script_header_length

    while (off < len_s) and (entry_type != S3BootScriptOpcode.EFI_BOOT_SCRIPT_TERMINATE_OPCODE):
        entry_type, s3script_entry = uefi_platform.parse_s3bootscript_entry(script_type, script, off, log_script)
        # couldn't parse the next entry - return what has been parsed so far
        if s3script_entry is None:
            return s3_boot_script_entries
        s3_boot_script_entries.append(s3script_entry)
        off += s3script_entry.length

    if log_script:
        logger().log('[uefi] +++ End of S3 Resume Boot-Script +++')

    logger().log_hal(f'[uefi] S3 Resume Boot-Script size: 0x{off:X}')
    logger().log_hal('\n[uefi] [++++++++++ S3 Resume Boot-Script Buffer ++++++++++]')
    if logger().HAL:
        print_buffer_bytes(script[: off])

    return s3_boot_script_entries


########################################################################################################
#
# UEFI Variables Parsing Functionality
#
########################################################################################################


EFI_VAR_NAME_PK = 'PK'
EFI_VAR_NAME_KEK = 'KEK'
EFI_VAR_NAME_db = 'db'
EFI_VAR_NAME_dbx = 'dbx'
EFI_VAR_NAME_SecureBoot = 'SecureBoot'
EFI_VAR_NAME_SetupMode = 'SetupMode'
EFI_VAR_NAME_CustomMode = 'CustomMode'
EFI_VAR_NAME_SignatureSupport = 'SignatureSupport'
EFI_VAR_NAME_certdb = 'certdb'
EFI_VAR_NAME_AuthVarKeyDatabase = 'AuthVarKeyDatabase'

#
# \MdePkg\Include\Guid\ImageAuthentication.h
#
# define EFI_IMAGE_SECURITY_DATABASE_GUID \
#  { \
#    0xd719b2cb, 0x3d3a, 0x4596, { 0xa3, 0xbc, 0xda, 0xd0, 0xe, 0x67, 0x65, 0x6f } \
#  }
#
# \MdePkg\Include\Guid\GlobalVariable.h
#
# define EFI_GLOBAL_VARIABLE \
#  { \
#    0x8BE4DF61, 0x93CA, 0x11d2, {0xAA, 0x0D, 0x00, 0xE0, 0x98, 0x03, 0x2B, 0x8C } \
#  }
#
EFI_GLOBAL_VARIABLE_GUID = '8be4df61-93ca-11d2-aa0d-00e098032b8c'
EFI_IMAGE_SECURITY_DATABASE_GUID = 'd719b2cb-3d3a-4596-a3bc-dad00e67656f'
# EFI_VAR_GUID_SecureBoot = EFI_GLOBAL_VARIABLE
# EFI_VAR_GUID_db         = EFI_IMAGE_SECURITY_DATABASE_GUID

EFI_VARIABLE_DICT: Dict[str, str] = {
    EFI_VAR_NAME_PK: EFI_GLOBAL_VARIABLE_GUID,
    EFI_VAR_NAME_KEK: EFI_GLOBAL_VARIABLE_GUID,
    EFI_VAR_NAME_db: EFI_IMAGE_SECURITY_DATABASE_GUID,
    EFI_VAR_NAME_dbx: EFI_IMAGE_SECURITY_DATABASE_GUID,
    EFI_VAR_NAME_SecureBoot: EFI_GLOBAL_VARIABLE_GUID,
    EFI_VAR_NAME_SetupMode: EFI_GLOBAL_VARIABLE_GUID,
    EFI_VAR_NAME_CustomMode: EFI_GLOBAL_VARIABLE_GUID,
    EFI_VAR_NAME_SignatureSupport: EFI_GLOBAL_VARIABLE_GUID
}


SECURE_BOOT_KEY_VARIABLES = (EFI_VAR_NAME_PK, EFI_VAR_NAME_KEK, EFI_VAR_NAME_db)
SECURE_BOOT_OPTIONAL_VARIABLES = (EFI_VAR_NAME_dbx,)
SECURE_BOOT_VARIABLES = (EFI_VAR_NAME_SecureBoot, EFI_VAR_NAME_SetupMode) + SECURE_BOOT_KEY_VARIABLES + SECURE_BOOT_OPTIONAL_VARIABLES
SECURE_BOOT_VARIABLES_ALL = (EFI_VAR_NAME_CustomMode, EFI_VAR_NAME_SignatureSupport) + SECURE_BOOT_VARIABLES
AUTHENTICATED_VARIABLES = (EFI_VAR_NAME_AuthVarKeyDatabase, EFI_VAR_NAME_certdb) + SECURE_BOOT_KEY_VARIABLES


def get_auth_attr_string(attr: int) -> str:
    attr_str = ' '
    if IS_VARIABLE_ATTRIBUTE(attr, EFI_VARIABLE_AUTHENTICATED_WRITE_ACCESS):
        attr_str = f'{attr_str}AWS+'
    if IS_VARIABLE_ATTRIBUTE(attr, EFI_VARIABLE_TIME_BASED_AUTHENTICATED_WRITE_ACCESS):
        attr_str = f'{attr_str}TBAWS+'
    if IS_VARIABLE_ATTRIBUTE(attr, EFI_VARIABLE_APPEND_WRITE):
        attr_str = f'{attr_str}AW+'
    return attr_str[:-1].lstrip()


def get_attr_string(attr: int) -> str:
    attr_str = ' '
    if IS_VARIABLE_ATTRIBUTE(attr, EFI_VARIABLE_NON_VOLATILE):
        attr_str = f'{attr_str}NV+'
    if IS_VARIABLE_ATTRIBUTE(attr, EFI_VARIABLE_BOOTSERVICE_ACCESS):
        attr_str = f'{attr_str}BS+'
    if IS_VARIABLE_ATTRIBUTE(attr, EFI_VARIABLE_RUNTIME_ACCESS):
        attr_str = f'{attr_str}RT+'
    if IS_VARIABLE_ATTRIBUTE(attr, EFI_VARIABLE_HARDWARE_ERROR_RECORD):
        attr_str = f'{attr_str}HER+'
    if IS_VARIABLE_ATTRIBUTE(attr, EFI_VARIABLE_AUTHENTICATED_WRITE_ACCESS):
        attr_str = f'{attr_str}AWS+'
    if IS_VARIABLE_ATTRIBUTE(attr, EFI_VARIABLE_TIME_BASED_AUTHENTICATED_WRITE_ACCESS):
        attr_str = f'{attr_str}TBAWS+'
    if IS_VARIABLE_ATTRIBUTE(attr, EFI_VARIABLE_APPEND_WRITE):
        attr_str = f'{attr_str}AW+'
    return attr_str[:-1].lstrip()


def print_efi_variable(offset: int, var_buf: bytes, var_header: 'EfiTableType', var_name: str, var_data: bytes, var_guid: str, var_attrib: int) -> None:
    logger().log('\n--------------------------------')
    logger().log(f'EFI Variable (offset = 0x{offset:X}):')
    logger().log('--------------------------------')

    # Print Variable Name
    logger().log(f'Name      : {var_name}')
    # Print Variable GUID
    logger().log(f'Guid      : {var_guid}')

    # Print Variable State
    if var_header:
        if 'State' in var_header._fields:
            state = getattr(var_header, 'State')
            state_str = 'State     :'
            if uefi_platform.IS_VARIABLE_STATE(state, uefi_platform.VAR_IN_DELETED_TRANSITION):
                state_str = f'{state_str} IN_DELETED_TRANSITION +'
            if uefi_platform.IS_VARIABLE_STATE(state, uefi_platform.VAR_DELETED):
                state_str = f'{state_str} DELETED +'
            if uefi_platform.IS_VARIABLE_STATE(state, uefi_platform.VAR_ADDED):
                state_str = f'{state_str} ADDED +'
            logger().log(state_str)

        # Print Variable Complete Header
        if logger().VERBOSE:
            if var_header.__str__:
                logger().log(str(var_header))
            else:
                decoded_header = uefi_platform.EFI_VAR_DICT[uefi_platform.FWType.EFI_FW_TYPE_UEFI]['name']
                logger().log(f'Decoded Header ({decoded_header}):')
                for attr in var_header._fields:
                    attr_str = f'{attr:<16}'
                    attr_value = getattr(var_header, attr)
                    logger().log(f'{attr_str} = {attr_value:X}')

    attr_str = (f'Attributes: 0x{var_attrib:X} ( {get_attr_string(var_attrib)} )')
    logger().log(attr_str)

    # Print Variable Data
    logger().log('Data:')
    print_buffer_bytes(var_data)

    # Print Variable Full Contents
    if logger().VERBOSE:
        logger().log('Full Contents:')
        if var_buf is not None:
            print_buffer_bytes(var_buf)


def print_sorted_EFI_variables(variables: Dict[str, List['EfiVariableType']]) -> None:
    sorted_names = sorted(variables.keys())
    rec: Tuple[int, bytes, EfiTableType, bytes, str, int]
    for name in sorted_names:
        for rec in variables[name]:
            #                   off,    buf,     hdr,         data,   guid,   attrs
            print_efi_variable(rec[0], rec[1], rec[2], name, rec[3], rec[4], rec[5])


def decode_EFI_variables(efi_vars: Dict[str, List['EfiVariableType']], nvram_pth: str) -> None:
    # print decoded and sorted EFI variables into a log file
    print_sorted_EFI_variables(efi_vars)
    # write each EFI variable into its own binary file
    for name in efi_vars.keys():
        n = 0
        data: bytes
        guid: str
        attrs: int
        for (_, _, _, data, guid, attrs) in efi_vars[name]:  # Type: EfiVariableType
            attr_str = get_attr_string(attrs)
            var_fname = os.path.join(nvram_pth, f'{name}_{guid}_{attr_str.strip()}_{n:d}.bin')
            write_file(var_fname, data)
            if name in SECURE_BOOT_KEY_VARIABLES:
                parse_efivar_file(var_fname, data, SECURE_BOOT_SIG_VAR)
            elif name == EFI_VAR_NAME_certdb:
                parse_efivar_file(var_fname, data, AUTH_SIG_VAR)
            elif name == EFI_VAR_NAME_AuthVarKeyDatabase:
                parse_efivar_file(var_fname, data, ESAL_SIG_VAR)
            n = n + 1


def identify_EFI_NVRAM(buffer: bytes) -> str:
    b = buffer
    for fw_type in uefi_platform.fw_types:
        if uefi_platform.EFI_VAR_DICT[fw_type]['func_getnvstore']:
            (offset, _, _) = uefi_platform.EFI_VAR_DICT[fw_type]['func_getnvstore'](b)
            if offset != -1:
                return fw_type
    return ''


def parse_EFI_variables(fname: str, rom: bytes, authvars: bool, _fw_type: Optional[str] = None) -> bool:
    if (_fw_type in uefi_platform.fw_types) and (_fw_type is not None):
        logger().log(f'[uefi] Using FW type (NVRAM format): {_fw_type}')
    else:
        logger().log_error(f"Unrecognized FW type '{_fw_type}' (NVRAM format) '{_fw_type}'.")
        return False

    logger().log('[uefi] Searching for NVRAM in the binary..')
    efi_vars_store = find_EFI_variable_store(rom, _fw_type)
    if efi_vars_store:
        nvram_fname = f'{fname}.nvram.bin'
        write_file(nvram_fname, efi_vars_store)
        nvram_pth = f'{fname}.nvram.dir'
        if not os.path.exists(nvram_pth):
            os.makedirs(nvram_pth)
        logger().log('[uefi] Extracting EFI Variables in the NVRAM..')
        efi_vars = uefi_platform.EFI_VAR_DICT[_fw_type]['func_getefivariables'](efi_vars_store)
        decode_EFI_variables(efi_vars, nvram_pth)
    else:
        logger().log_error('Did not find NVRAM')
        return False

    return True


def find_EFI_variable_store(rom_buffer: Optional[bytes], _FWType: Optional[str]) -> bytes:
    if rom_buffer is None:
        logger().log_error('rom_buffer is None')
        return b''

    rom = rom_buffer
    offset = 0
    size = len(rom_buffer)
    nvram_header = None
    if _FWType is None:
        logger().log_hal(f'[uefi] find_EFI_variable_store(): _FWType is None. Bypassing find_EFI_variable_store().')
        return b''
    if uefi_platform.EFI_VAR_DICT[_FWType]['func_getnvstore']:
        (offset, size, nvram_header) = uefi_platform.EFI_VAR_DICT[_FWType]['func_getnvstore'](rom)
        if (-1 == offset):
            logger().log_error("'func_getnvstore' is defined but could not find EFI NVRAM. Exiting..")
            return b''
    else:
        logger().log("[uefi] 'func_getnvstore' is not defined in EFI_VAR_DICT. Assuming start offset 0.")

    if -1 == size:
        size = len(rom_buffer)
    nvram_buf = rom[offset: offset + size]

    if logger().UTIL_TRACE:
        logger().log(f'[uefi] Found EFI NVRAM at offset 0x{offset:08X}')
        logger().log("""
==================================================================
NVRAM: EFI Variable Store
==================================================================""")
        if nvram_header:
            logger().log(nvram_header)
    return nvram_buf

########################################################################################################
#
# UEFI HAL Component
#
########################################################################################################


class UEFI(hal_base.HALBase):
    def __init__(self, cs):
        super(UEFI, self).__init__(cs)
        self.helper = cs.helper
        # if cs is not None:
        #    self.cs = cs
        #    self.helper = cs.helper
        # else:
        #    self.helper = helper
        self._FWType = uefi_platform.FWType.EFI_FW_TYPE_UEFI

    ######################################################################
    # FWType defines platform/BIOS dependent formats like
    # format of EFI NVRAM, format of FV, etc.
    #
    # FWType chooses an element from the EFI_VAR_DICT Dictionary
    #
    # Default current platform type is EFI_FW_TYPE_UEFI
    ######################################################################

    def set_FWType(self, efi_nvram_format: str) -> None:
        if efi_nvram_format in uefi_platform.fw_types:
            self._FWType = efi_nvram_format

    ######################################################################
    # EFI NVRAM Parsing Functions
    ######################################################################

    def init_spi_hal(self) -> None:
        if not hasattr(self, 'spi'):
            self.spi = SPI(self.cs)

    def dump_EFI_variables_from_SPI(self) -> bytes:
        self.init_spi_hal()
        (_, limit, _) = self.spi.get_SPI_region(BIOS)
        spi_size = limit + 1
        self.logger.log_hal(f'[uefi] Reading from SPI: 0x0-0x{spi_size:X}')
        return self.read_EFI_variables_from_SPI(0, spi_size)

    def read_EFI_variables_from_rom(self, rom: bytes) -> bytes:
        self.logger.log_hal('[uefi] Looking for variables in SPI dump')
        efi_var_store = find_EFI_variable_store(rom, 'nvar')
        if efi_var_store:
            efi_vars = uefi_platform.EFI_VAR_DICT['nvar']['func_getefivariables'](efi_var_store)
            return efi_vars
        return efi_var_store

    def read_EFI_variables_from_SPI(self, BIOS_region_base: int, BIOS_region_size: int) -> bytes:
        self.init_spi_hal()
        rom = self.spi.read_spi(BIOS_region_base, BIOS_region_size)
        return self.read_EFI_variables_from_rom(rom)

    def read_EFI_variables_from_file(self, filename: str) -> bytes:
        rom = read_file(filename)
        return self.read_EFI_variables_from_rom(rom)

    # @TODO: Do not use, will be removed

    def read_EFI_variables(self, efi_var_store: Optional[bytes], authvars: bool) -> Dict[str, List['EfiVariableType']]:
        if efi_var_store is None:
            logger().log_error('efi_var_store is None')
            return {}
        variables: Dict[str, List[EfiVariableType]] = uefi_platform.EFI_VAR_DICT[self._FWType]['func_getefivariables'](efi_var_store)
        if logger().UTIL_TRACE:
            print_sorted_EFI_variables(variables)
        return variables

    ######################################################################
    # S3 Resume Boot-Script Parsing Functions
    ######################################################################

    #
    # Finds physical address of the S3 resume boot script from UEFI variables
    # Returns:
    #   found               - status is the script is found
    #   AcpiBootScriptTable - physical address of the S3 resume boot script, 0 if (not found)
    #
    def find_s3_bootscript(self) -> Tuple[bool, List[int]]:
        found = False
        BootScript_addresses = []

        efivars = self.list_EFI_variables()
        if efivars is None:
            logger().log_error('Could not enumerate UEFI variables at runtime')
            return (found, BootScript_addresses)
        logger().log_hal(f'[uefi] Searching for EFI variable(s): {str(S3_BOOTSCRIPT_VARIABLES)}')

        for efivar_name in efivars:
            (off, buf, hdr, data, guid, attrs) = efivars[efivar_name][0]
            if efivar_name in S3_BOOTSCRIPT_VARIABLES:
                logger().log_hal(f'[uefi] Found: {efivar_name} {{{guid}}} {get_attr_string(attrs)} variable')
                logger().log_hal(f'[uefi] {efivar_name} variable data:')
                if logger().HAL:
                    print_buffer_bytes(data)

                varsz = len(data)
                if 4 == varsz:
                    AcpiGlobalAddr_fmt = '<L'
                elif 8 == varsz:
                    AcpiGlobalAddr_fmt = '<Q'
                else:
                    logger().log_error(f"Unrecognized format of '{efivar_name}' UEFI variable (data size = 0x{varsz:X})")
                    break
                AcpiGlobalAddr = struct.unpack_from(AcpiGlobalAddr_fmt, data)[0]
                if 0 == AcpiGlobalAddr:
                    logger().log_error(f'Pointer to ACPI Global Data structure in {efivar_name} variable is 0')
                    break
                logger().log_hal(f"[uefi] Pointer to ACPI Global Data structure: 0x{AcpiGlobalAddr:016X}")
                logger().log_hal('[uefi] Decoding ACPI Global Data structure...')
                AcpiVariableSet = self.helper.read_phys_mem(AcpiGlobalAddr, ACPI_VARIABLE_SET_STRUCT_SIZE)
                logger().log_hal('[uefi] AcpiVariableSet structure:')
                if logger().HAL:
                    print_buffer_bytes(AcpiVariableSet)
                AcpiVariableSet_fmt = '<6Q'
                # if len(AcpiVariableSet) < struct.calcsize(AcpiVariableSet_fmt):
                #    logger().log_error( 'Unrecognized format of AcpiVariableSet structure' )
                #    return (False,0)
                _, _, _, AcpiBootScriptTable, _, _ = struct.unpack_from(AcpiVariableSet_fmt, AcpiVariableSet)
                logger().log_hal(f'[uefi] ACPI Boot-Script table base = 0x{AcpiBootScriptTable:016X}')
                found = True
                BootScript_addresses.append(AcpiBootScriptTable)
                # break
        return (found, BootScript_addresses)

    #
    # Upper level function to find and parse S3 resume boot script
    # Returns:
    #   bootscript_pa  - physical address of the S3 resume boot script
    #   script_entries - a list of parse S3 resume boot script operations
    #
    def get_s3_bootscript(self, log_script: bool = False) -> Tuple[List[int], Optional[Dict[int, List['S3BOOTSCRIPT_ENTRY']]]]:
        parsed_scripts = {}
        script_entries = []
        #
        # Find the S3 Resume Boot-Script from UEFI variables
        #
        found, bootscript_PAs = self.find_s3_bootscript()
        if not found:
            return (bootscript_PAs, None)
        logger().log_hal(f'[uefi] Found {len(bootscript_PAs):d} S3 resume boot-scripts')

        for bootscript_pa in bootscript_PAs:
            if (bootscript_pa == 0):
                continue
            logger().log_hal(f'[uefi] S3 resume boot-script at 0x{bootscript_pa:016X}')
            #
            # Decode the S3 Resume Boot-Script into a sequence of operations/opcodes
            #
            # @TODO: should be dumping memory contents in a loop until end opcode is found or id'ing actual size
            script_buffer = self.helper.read_phys_mem(bootscript_pa, 0x200000)
            logger().log_hal('[uefi] Decoding S3 Resume Boot-Script...')
            script_entries = parse_script(script_buffer, log_script)
            parsed_scripts[bootscript_pa] = script_entries
        return (bootscript_PAs, parsed_scripts)

    ######################################################################
    # Runtime Variable API Functions
    ######################################################################

    def list_EFI_variables(self) -> Optional[Dict[str, List[Tuple[int, bytes, int, bytes, str, int]]]]:
        varlist = self.helper.list_EFI_variables()
        return varlist

    def list_EFI_variables_spi(self, filename: str = None) -> Optional[Dict[str, List[Tuple[int, bytes, int, bytes, str, int]]]]:
        if filename:
            varlist = self.read_EFI_variables_from_file(filename)
        else:
            varlist = self.dump_EFI_variables_from_SPI()
        return varlist

    def get_EFI_variable(self, name: str, guid: str, filename: Optional[str] = None) -> Optional[bytes]:
        var = self.helper.get_EFI_variable(name, guid)
        if var:
            if filename:
                write_file(filename, var)
            if logger().UTIL_TRACE or logger().HAL:
                logger().log(f'[uefi] EFI variable {guid}:{name} :')
                print_buffer_bytes(var)
        return var

    def set_EFI_variable(self, name: str, guid: str, var: bytes, datasize: Optional[int] = None, attrs: Optional[int] = None) -> Optional[int]:
        atts_str = '' if attrs is None else f'(attributes = {attrs})'
        logger().log_hal(f'[uefi] Writing EFI variable {guid}:{name} {atts_str}')
        return self.helper.set_EFI_variable(name, guid, var, datasize, attrs)

    def set_EFI_variable_from_file(self, name: str, guid: str, filename: str, datasize: Optional[int] = None, attrs: Optional[int] = None) -> Optional[int]:
        if filename is None:
            logger().log_error('File with EFI variable is not specified')
            return False
        var = read_file(filename)
        return self.set_EFI_variable(name, guid, var, datasize, attrs)

    def delete_EFI_variable(self, name: str, guid: str) -> Optional[int]:
        logger().log_hal(f'[uefi] Deleting EFI variable {guid}:{name}')
        return self.helper.delete_EFI_variable(name, guid)

    ######################################################################
    # UEFI System Tables
    ######################################################################

    EfiTable = Tuple[bool, int, Optional[EFI_TABLE_HEADER], Optional['EFI_SYSTEM_TABLE'], bytes]

    def find_EFI_Table(self, table_sig: str) -> EfiTable:
        (smram_base, _, _) = self.cs.cpu.get_SMRAM()
        CHUNK_SZ = 1024 * 1024  # 1MB
        logger().log_hal(f"[uefi] Searching memory for EFI table with signature '{table_sig}'...")
        table_pa = 0
        table_header = None
        table = None
        table_buf = b''
        pa = smram_base - CHUNK_SZ
        isFound = False

        (tseg_base, tseg_limit, _) = self.cs.cpu.get_TSEG()

        while pa > CHUNK_SZ:
            if (pa <= tseg_limit) and (pa >= tseg_base):
                logger().log_hal(f'[uefi] Skipping memory read at pa: {pa:016X}')
                pa -= CHUNK_SZ
                continue
            logger().log_hal(f'[uefi] Reading 0x{pa:016X}...')
            try:
                membuf = self.cs.mem.read_physical_mem(pa, CHUNK_SZ)
            except OsHelperError as err:
                logger().log_hal(f'[uefi] Unable to read memory at pa: {pa:016X} Error: {err}')
                pa -= CHUNK_SZ
                continue
            pos = bytestostring(membuf).find(table_sig)
            if -1 != pos:
                table_pa = pa + pos
                logger().log_hal(f"[uefi] Round signature '{table_sig}' at 0x{table_pa:016X}...")
                if pos < (CHUNK_SZ - EFI_TABLE_HEADER_SIZE):
                    hdr = membuf[pos: pos + EFI_TABLE_HEADER_SIZE]
                else:
                    hdr = self.cs.mem.read_physical_mem(table_pa, EFI_TABLE_HEADER_SIZE)
                table_header = EFI_TABLE_HEADER(*struct.unpack_from(EFI_TABLE_HEADER_FMT, hdr))
                # do some sanity checks on the header
                is_reserved = table_header.Reserved != 0
                is_bad_crc = table_header.CRC32 == 0
                is_not_table_rev = table_header.Revision not in EFI_REVISIONS
                is_not_correct_size = table_header.HeaderSize > MAX_EFI_TABLE_SIZE
                if is_reserved or is_bad_crc or is_not_table_rev or is_not_correct_size:
                    logger().log_hal(f"[uefi] Found '{table_sig}' at 0x{table_pa:016X} but doesn't look like an actual table. Keep searching...")
                    logger().log_hal(str(table_header))
                else:
                    isFound = True
                    logger().log_hal(f"[uefi] Found EFI table at 0x{table_pa:016X} with signature '{table_sig}'...")
                    table_size = struct.calcsize(EFI_TABLES[table_sig]['fmt'])
                    if pos < (CHUNK_SZ - EFI_TABLE_HEADER_SIZE - table_size):
                        table_buf = membuf[pos: pos + EFI_TABLE_HEADER_SIZE + table_size]
                    else:
                        table_buf = self.cs.mem.read_physical_mem(table_pa, EFI_TABLE_HEADER_SIZE + table_size)
                    table = EFI_TABLES[table_sig]['struct'](*struct.unpack_from(EFI_TABLES[table_sig]['fmt'], table_buf[EFI_TABLE_HEADER_SIZE:]))
                    if logger().HAL:
                        print_buffer_bytes(table_buf)
                    logger().log_hal(f'[uefi] {EFI_TABLES[table_sig]["name"]}:')
                    logger().log_hal(str(table_header))
                    logger().log_hal(str(table))
                    break
            pa -= CHUNK_SZ
        if not isFound:
            logger().log_hal(f"[uefi] Could not find EFI table with signature '{table_sig}'")
        return (isFound, table_pa, table_header, table, table_buf)

    def find_EFI_System_Table(self) -> EfiTable:
        return self.find_EFI_Table(EFI_SYSTEM_TABLE_SIGNATURE)

    def find_EFI_RuntimeServices_Table(self) -> EfiTable:
        return self.find_EFI_Table(EFI_RUNTIME_SERVICES_SIGNATURE)

    def find_EFI_BootServices_Table(self) -> EfiTable:
        return self.find_EFI_Table(EFI_BOOT_SERVICES_SIGNATURE)

    def find_EFI_DXEServices_Table(self) -> EfiTable:
        return self.find_EFI_Table(EFI_DXE_SERVICES_TABLE_SIGNATURE)
    # def find_EFI_PEI_Table( self ):
    #    return self.find_EFI_Table( EFI_FRAMEWORK_PEI_SERVICES_TABLE_SIGNATURE )
    # def find_EFI_SMM_System_Table( self ):
    #    return self.find_EFI_Table( EFI_SMM_SYSTEM_TABLE_SIGNATURE )

    def find_EFI_Configuration_Table(self) -> Tuple[bool, int, Optional[EFI_CONFIGURATION_TABLE], bytes]:
        ect_pa = 0
        ect = None
        ect_buf = b''
        (isFound, _, _, est, _) = self.find_EFI_System_Table()
        if isFound and est is not None:
            if 0 != est.BootServices:
                logger().log_hal('[uefi] UEFI appears to be in Boot mode')
                ect_pa = est.ConfigurationTable
            else:
                logger().log_hal('[uefi] UEFI appears to be in Runtime mode')
                ect_pa = self.cs.mem.va2pa(est.ConfigurationTable)
                if not ect_pa:
                    # Most likely the VA in the System Table is not mapped so find the RST by signature and
                    # then compute the address of the configuration table.  This assumes the VA mapping keeps
                    # the pages in the same relative location as in physical memory.
                    (rst_found, rst_pa, rst_header, rst, rst_buf) = self.find_EFI_RuntimeServices_Table()
                    if rst_found:
                        if logger().HAL:
                            logger().log_warning('Attempting to derive configuration table address')
                        ect_pa = rst_pa + (est.ConfigurationTable - est.RuntimeServices)
                    else:
                        if logger().HAL:
                            logger().log_warning("Can't find UEFI ConfigurationTable")
                        return (False, ect_pa, ect, ect_buf)
        if est is not None:
            logger().log_hal(f'[uefi] EFI Configuration Table ({est.NumberOfTableEntries:d} entries): VA = 0x{est.ConfigurationTable:016X}, PA = 0x{ect_pa:016X}')
        else:
            logger().log_hal(f'[uefi] EFI Configuration Table (No entries found)')

        found = ect_pa is not None
        if found and (est is not None):
            ect_buf = self.cs.mem.read_physical_mem(ect_pa, EFI_VENDOR_TABLE_SIZE * est.NumberOfTableEntries)
            ect = EFI_CONFIGURATION_TABLE()
            for i in range(est.NumberOfTableEntries):
                vt = EFI_VENDOR_TABLE(*struct.unpack_from(EFI_VENDOR_TABLE_FORMAT, ect_buf[i * EFI_VENDOR_TABLE_SIZE:]))
                ect.VendorTables[vt.VendorGuid()] = vt.VendorTable
        return (found, ect_pa, ect, ect_buf)

    def dump_EFI_tables(self) -> None:
        (found, pa, hdr, table, table_buf) = self.find_EFI_System_Table()
        if found:
            logger().log('[uefi] EFI System Table:')
            if table_buf is not None:
                print_buffer_bytes(table_buf)
            logger().log(str(hdr))
            logger().log(str(table))
        (found, _, ect, ect_buf) = self.find_EFI_Configuration_Table()
        if found:
            logger().log('\n[uefi] EFI Configuration Table:')
            if ect_buf is not None:
                print_buffer_bytes(ect_buf)
            logger().log(str(ect))
        (found, pa, hdr, table, table_buf) = self.find_EFI_RuntimeServices_Table()
        if found:
            logger().log('\n[uefi] EFI Runtime Services Table:')
            if table_buf is not None:
                print_buffer_bytes(table_buf)
            logger().log(str(hdr))
            logger().log(str(table))
        (found, pa, hdr, table, table_buf) = self.find_EFI_BootServices_Table()
        if found:
            logger().log('\n[uefi] EFI Boot Services Table:')
            if table_buf is not None:
                print_buffer_bytes(table_buf)
            logger().log(str(hdr))
            logger().log(str(table))
        (found, pa, hdr, table, table_buf) = self.find_EFI_DXEServices_Table()
        if found:
            logger().log('\n[uefi] EFI DXE Services Table:')
            if table_buf is not None:
                print_buffer_bytes(table_buf)
            logger().log(str(hdr))
            logger().log(str(table))



#!/usr/bin/env python3
# CHIPSEC: Platform Security Assessment Framework
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; Version 2.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
#
# Contact information:
# chipsec@intel.com
#
"""Launch a QEMU virtual machine with a UEFI Shell and chipsec

This scripts uses the UEFI Shell provided by OVMF firmware. Therefore it
requires both QEMU (for x86_64 system emulation) and OVMF to be installed.
It also requires qemu-img, to create a temporary boot disk.

On Debian and Ubuntu, installing these requirements can be done with:

    sudo apt install ovmf qemu-system-x86 qemu-utils

Usage examples:

    # Run chipsec_main.py in the default QEMU virtual machine
    qemu_efi.py -m

    # Enumerate PCI devices of the virtual machine (like command "pci" of UEFI shell)
    qemu_efi.py -u pci enumerate

    # Launch a Python interpreter where it is possible to import chipsec and edk2
    qemu_efi.py -p
"""
import argparse
from pathlib import Path
import shlex
import shutil
import subprocess
import sys
import tempfile
import zipfile


CHIPSEC_BASE = Path(__file__).parent / ".."
PYTHON_INSTALL_ZIP = CHIPSEC_BASE / "__install__" / "UEFI" / "chipsec_py368_uefi_x64.zip"
PYTHON_EFI_COMMAND = "python.efi"

OVMF_CODE_LOCATIONS = (
    # Arch Linux https://archlinux.org/packages/extra/any/edk2-ovmf/
    "/usr/share/edk2-ovmf/x64/OVMF_CODE.fd",
    # Debian https://packages.debian.org/fr/sid/all/ovmf/filelist
    "/usr/share/OVMF/OVMF_CODE.fd",
    # Fedora https://fedora.pkgs.org/35/fedora-updates-x86_64/edk2-ovmf-20211126gitbb1bba3d7767-1.fc35.noarch.rpm.html
    "/usr/share/edk2/ovmf/OVMF_CODE.fd",
)


def create_efi_disk(filename, startup_nsh=None):
    """Create a disk bootable through UEFI with chipsec

    This function copies all needed files to a temporary directory and uses
    qemu-img to convert this directory into a FAT partition.
    """
    with tempfile.TemporaryDirectory(prefix="efidisk_") as tmpdir:
        # Unzip Python into the temporary directory
        print("Extracting {} to {} ...".format(PYTHON_INSTALL_ZIP.name, tmpdir))
        with zipfile.ZipFile(PYTHON_INSTALL_ZIP) as python_zip:
            python_zip.extractall(tmpdir)

        # Copy Chipsec
        print("Copying chipsec to {} ...".format(tmpdir))
        for extension in (".py", ".xml", ".xsd"):
            for filepath in CHIPSEC_BASE.glob("**/*" + extension):
                destination = Path(tmpdir) / "chipsec" / filepath.relative_to(CHIPSEC_BASE)
                destination.parent.mkdir(parents=True, exist_ok=True)
                shutil.copy(filepath, destination)

        # Create startup.nsh
        if startup_nsh:
            with (Path(tmpdir) / "startup.nsh").open("w") as f:
                f.write(startup_nsh)

        # Convert to QCow image
        print("Converting to QCow2 format in {} ...".format(filename))
        subprocess.run(
            ("qemu-img", "convert", "-c", "-f", "vvfat", "-O", "qcow2", "fat:" + tmpdir, filename),
            check=True
        )


def unescape_arguments(args):
    """Replace " -" with "-" in the arguments"""
    return [arg[1:] if arg.startswith(" -") else arg for arg in args]


def main():
    parser = argparse.ArgumentParser(description="Launch a QEMU virtual machine with a UEFI Shell and chipsec")
    group = parser.add_mutually_exclusive_group()
    group.add_argument("-m", "--main", nargs="*", type=str,
                       help="start chipsec_main.py, optionally with some arguments")
    group.add_argument("-p", "--python", nargs="*", type=str,
                       help="start a Python-UEFI shell, optionally with some arguments")
    group.add_argument("-u", "--util", nargs="+", type=str,
                       help="start chipsec_util.py, with some arguments (use ' -' to escape the dash prefix)")
    parser.add_argument("--ovmf", type=str,
                        help="path to the OVMF firmware code")
    parser.add_argument("--memory", type=str, default="512M",
                        help="set the RAM size of the virtual machine (QEMU option -m)")
    parser.add_argument("-g", "--gui", action="store_true",
                        help="use QEMU graphics window")
    parser.add_argument("-H", "--host-cpu", action="store_true",
                        help="use host CPU")
    parser.add_argument("-M", "--machine", type=str,
                        help="specify QEMU machine ('pc', 'q35,smm=on'...)")
    parser.add_argument("-Q", "--qemu-arg", nargs="+", type=str,
                        help="specify additional QEMU arguments (use ' -' to escape the dash prefix)")
    args = parser.parse_args()

    ovmf_path = args.ovmf
    if not ovmf_path:
        for test_ovmf_path in OVMF_CODE_LOCATIONS:
            if Path(test_ovmf_path).exists():
                ovmf_path = test_ovmf_path
                break
        if not ovmf_path:
            print("No OVMF file found. You can specify one using option --ovmf.", file=sys.stderr)
            sys.exit(1)

    # Create startup.nsh
    start_cmd = None
    if args.main is not None:
        start_cmd = [PYTHON_EFI_COMMAND, "chipsec_main.py"] + unescape_arguments(args.main)
    elif args.python is not None:
        start_cmd = [PYTHON_EFI_COMMAND] + unescape_arguments(args.python)
    elif args.util is not None:
        start_cmd = [PYTHON_EFI_COMMAND, "chipsec_util.py"] + unescape_arguments(args.util)

    if start_cmd:
        startup_nsh = "fs0:\ncd chipsec\n" + " ".join(shlex.quote(arg) for arg in start_cmd) + "\nreset\n"
    else:
        startup_nsh = "@echo Type 'reset' to exit"

    qemu_cmd = [
        "qemu-system-x86_64",
        # Make OVMF boot directly in UEFI shell instead of trying network boot (PXE)
        "-nic", "none",
        # Enable exiting by entering "reset" in UEFI shell
        "-no-reboot",
        # Start OVMF
        "-drive", "if=pflash,format=raw,readonly=on,file=" + ovmf_path,
    ]
    if not args.gui:
        qemu_cmd += ["-nographic"]

    if args.memory:
        qemu_cmd += ["-m", args.memory]

    if args.host_cpu:
        qemu_cmd += ["-cpu", "host", "-enable-kvm"]

    if args.machine:
        qemu_cmd += ["-machine", args.machine]

    with tempfile.NamedTemporaryFile(prefix="efidisk_") as tmpdisk:
        create_efi_disk(tmpdisk.name, startup_nsh)
        qemu_cmd += ["-drive", "format=qcow2,file=" + tmpdisk.name]

        if args.qemu_arg:
            qemu_cmd += [arg[1:] if arg.startswith(" ") else arg for arg in args.qemu_arg]

        print("Launching {}".format(" ".join(qemu_cmd)))
        subprocess.run(qemu_cmd, check=True)


if __name__ == "__main__":
    main()



#CHIPSEC: Platform Security Assessment Framework
#Copyright (c) 2010-2021, Intel Corporation
#
#This program is free software; you can redistribute it and/or
#modify it under the terms of the GNU General Public License
#as published by the Free Software Foundation; Version 2.
#
#This program is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.
#
#You should have received a copy of the GNU General Public License
#along with this program; if not, write to the Free Software
#Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
#
#Contact information:
#chipsec@intel.com
#

import os

doc_path = os.getcwd()
src_path = os.path.abspath(os.path.join(doc_path, '..', '..'))

# Remove subheaders from all rst modules
def modulesRst():
    for _, _, rstFiles in os.walk(os.path.join(doc_path, 'modules')):
        for file in rstFiles:
            f = open(os.path.join('modules', file), 'r')
            title = f.readline()
            contents = f.read()
            f.close()

            contents = contents.replace('Submodules\n----------\n\n', '')
            contents = contents.replace('Subpackages\n-----------\n\n', '')
            contents = contents.replace('Module contents\n---------------\n\n', '')
            contents = contents.replace(':show-inheritance:', '')
            contents = contents.replace(':undoc-members:', '')
            contents = contents.replace(':members:', '')
            
            f = open(os.path.join('modules', file), 'w')
            f.write(title + contents)
            f.close()

# Create rst for xml files
def xmlRst():
    moduleStr = ''
    for _, _, cfgFiles in os.walk(os.path.join(src_path, 'chipsec', 'cfg', '8086')):
        for cfg in cfgFiles:
            if ".py" not in cfg: 
                try:
                    with open(os.path.join(src_path, 'chipsec', 'cfg', '8086', cfg), 'r') as f:
                        xmlContent = f.read()
                except Exception:
                    print(f'\t\tUnable to remove {cfg}!!!')
                else:
                    moduleStr += '\tchipsec.cfg.8086.{0}.rst\n'.format(cfg)
                    commentBegins = xmlContent.find('<!--')
                    commentEnds = xmlContent.find('-->')
                    xmlComment = xmlContent[commentBegins+4:commentEnds] + '\n'
                    
                    with open(os.path.join(doc_path, 'modules', 'chipsec.cfg.8086.' + cfg + '.rst'), 'w') as f:
                        path = "chipsec\\\\cfg\\\\8086\\\\" + cfg
                        f.write( cfg[:-4] + "\n" + "=" *len(cfg) + "\n\n" + "Path: " + path + "\n\n" + xmlComment )
                        
        with open(os.path.join(doc_path, 'modules', 'chipsec.cfg.8086.rst'), 'w') as f:
            f.write(".. toctree::\n\n" + moduleStr)

if __name__ == "__main__":
    modulesRst()
    xmlRst()



# CHIPSEC: Platform Security Assessment Framework
# Copyright (c) 2019, Eclypsium, Inc.
# Copyright (c) 2019-2021, Intel Corporation
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; Version 2.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

"""
This module checks that SPD Write Disable bit in SMBus controller has been set

References:
    Intel 8 Series/C220 Series Chipset Family Platform Controller Hub datasheet
    Intel 300 Series Chipset Families Platform Controller Hub datasheet

This module checks the following:

    SMBUS_HCFG.SPD_WD

The module returns the following results:

    PASSED : SMBUS_HCFG.SPD_WD is set

    FAILED : SMBUS_HCFG.SPD_WD is not set and SPDs were detected

    INFORMATION: SMBUS_HCFG.SPD_WD is not set, but no SPDs were detected

Hardware registers used:

    SMBUS_HCFG

Usage:
    ``chipsec_main -m common.spd_wd``

Examples:
    >>> chipsec_main.py -m common.spd_wd

.. NOTE::
    This module will only run if:
        - SMBUS device is enabled
        - SMBUS_HCFG.SPD_WD is defined for the platform
"""

from chipsec.module_common import BaseModule, BIOS
from chipsec.library.returncode import ModuleResult
from chipsec.hal.smbus import SMBus
from chipsec.hal.spd import SPD
from typing import List

TAGS = [BIOS]
METADATA_TAGS = ['OPENSOURCE', 'IA', 'COMMON', 'SPD_WD']


class spd_wd(BaseModule):

    def __init__(self):
        BaseModule.__init__(self)

    def is_supported(self) -> bool:
        if self.cs.device.is_enabled('SMBUS'):
            if self.cs.register.has_field('SMBUS_HCFG', 'SPD_WD'):
                return True
            else:
                self.logger.log_important('SMBUS_HCFG.SPD_WD is not defined for this platform.  Skipping module.')
        else:
            self.logger.log_important('SMBUS device appears disabled.  Skipping module.')
        return False

    def check_spd_wd(self) -> int:
        try:
            _smbus = SMBus(self.cs)
            _spd = SPD(_smbus)
        except BaseException as msg:
            self.logger.log_error(msg)
            self.result.setStatusBit(self.result.status.INFORMATION)
            self.res = self.result.getReturnCode(ModuleResult.ERROR)
            return self.res

        spd_wd_reg = self.cs.register.read('SMBUS_HCFG')
        spd_wd = self.cs.register.get_field('SMBUS_HCFG', spd_wd_reg, 'SPD_WD')

        self.cs.register.print('SMBUS_HCFG', spd_wd_reg)

        if 1 == spd_wd:
            self.logger.log_passed('SPD Write Disable is set')
            self.res = ModuleResult.PASSED
        else:
            if _spd.detect():
                self.logger.log_failed('SPD Write Disable is not set and SPDs were detected')
                self.result.setStatusBit(self.result.status.POTENTIALLY_VULNERABLE)
                self.res = ModuleResult.FAILED
            else:
                self.logger.log_information('SPD Write Disable is not set, but no SPDs detected')
                self.result.setStatusBit(self.result.status.INFORMATION)
                self.res = ModuleResult.INFORMATION

        return self.result.getReturnCode(self.res)

    def run(self, module_argv: List[str]) -> int:
        self.logger.start_test('SPD Write Disable')
        self.logger.log('')
        return self.check_spd_wd()



# !/usr/bin/python
# CHIPSEC: Platform Security Assessment Framework
# Copyright (c) 2021, Intel Corporation
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; Version 2.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

# Contact information:
# chipsec@intel.com

"""
>>> chipsec_util config show [config] <name>

Examples:

>>> chipsec_util config show ALL
>>> chipsec_util config show MMIO_BARS
>>> chipsec_util config show REGISTERS BC
"""

from argparse import ArgumentParser

from chipsec.command import BaseCommand, toLoad
from typing import Any, Dict


class CONFIGCommand(BaseCommand):

    def requirements(self) -> toLoad:
        return toLoad.All

    def parse_arguments(self) -> None:
        parser = ArgumentParser(usage=__doc__)

        subparsers = parser.add_subparsers()

        # show
        parser_show = subparsers.add_parser('show')
        parser_show.add_argument('config', choices=['CONFIG_PCI', 'REGISTERS', 'MMIO_BARS', 'IO_BARS', 'MEMORY_RANGES', 'CONTROLS', 'BUS', 'LOCKS', 'ALL'])
        parser_show.add_argument('name', type=str, nargs='*', help="Specific Name", default=[])
        parser_show.set_defaults(func=self.show, config="ALL")

        parser.parse_args(self.argv, namespace=self)

    def show(self) -> None:
        if self.config == "ALL":
            config = ['CONFIG_PCI', 'REGISTERS', 'MMIO_BARS', 'IO_BARS', 'MEMORY_RANGES', 'CONTROLS', 'BUS', 'LOCKS']
        else:
            config = [self.config]
        for mconfig in config:
            cfg = getattr(self.cs.Cfg, mconfig)
            if not self.name or len(config) > 1:
                self.name = sorted(cfg.keys())
            self.logger.log(mconfig)
            for name in self.name:
                if mconfig == "REGISTERS":
                    self.logger.log(f'\t{name} - {self.register_details(cfg[name])}')
                elif mconfig == "CONFIG_PCI":
                    self.logger.log(f'\t{name} - {self.pci_details(cfg[name])}')
                elif mconfig == "MMIO_BARS":
                    self.logger.log(f'\t{name} - {self.mmio_details(cfg[name])}')
                elif mconfig == "IO_BARS":
                    self.logger.log(f'\t{name} - {self.io_details(cfg[name])}')
                elif mconfig == "MEMORY_RANGES":
                    self.logger.log(f'\t{name} - {self.mem_details(cfg[name])}')
                elif mconfig == "CONTROLS":
                    self.logger.log(f'\t{name} - {self.control_details(cfg[name])}')
                elif mconfig == "LOCKS":
                    self.logger.log(f'\t{name} - {self.lock_details(cfg[name])}')
                elif mconfig == "BUS":
                    self.logger.log(f'\t{name} - {self.bus_details(cfg[name])}')

    def register_details(self, regi: Dict[str, Any]) -> str:
        ret = ''
        if regi['type'] == 'pcicfg' or regi['type'] == 'mmcfg':
            if 'device' in regi.keys():
                ret = f'device: {regi["device"]}, offset: {regi["offset"]}, size: {regi["size"]}'
            else:
                ret = f'bus: {regi["bus"]}, dev: {regi["dev"]}, func: {regi["fun"]}, offset: {regi["offset"]}, size: {regi["size"]}'
        elif regi['type'] == 'mmio':
            ret = f'bar: {regi["bar"]}, offset: {regi["offset"]}, size: {regi["size"]}'
        elif regi['type'] == 'mm_msgbus':
            ret = f'port: {regi["port"]}, offset: {regi["offset"]}, size: {regi["size"]}'
        elif regi['type'] == 'io':
            ret = f'port: {regi["port"]}, size: {regi["size"]}'
        elif regi['type'] == 'iobar':
            ret = f'bar: {regi["bar"]}, offset: {regi["offset"]}, size: {regi["size"]}'
        elif regi['type'] == 'msr':
            ret = f'msr: {regi["msr"]}, size: {regi["size"]}'
        elif regi['type'] == 'R Byte':
            ret = f'offset: {regi["offset"]}, size: {regi["size"]}'
        elif regi['type'] == 'memory':
            ret = f'access: {regi["access"]}, address: {regi["address"]}, offset: {regi["offset"]}, size: {regi["size"]}'
        if 'FIELDS' in regi.keys():
            for key in regi['FIELDS'].keys():
                extension = (f'\n\t\t{key} - bit {regi["FIELDS"][key]["bit"]}:{int(regi["FIELDS"][key]["size"]) + int(regi["FIELDS"][key]["bit"]) - 1}')
                ret += extension
        return ret

    def pci_details(self, regi: Dict[str, Any]) -> str:
        ret = f'bus: {regi["bus"]}, dev: {regi["dev"]}, func: {regi["fun"]}, vid: {regi["vid"]}, did: {regi["did"] if "did" in regi.keys() else None}'
        return ret

    def mmio_details(self, regi: Dict[str, Any]) -> str:
        regi_size = regi['size'] if 'size' in regi.keys() else None
        fixed_addr = regi['fixed_address'] if 'fixed_address' in regi.keys() else None
        if 'register' in regi.keys():
            ret = f'register: {regi["register"]}, base_field: {regi["base_field"]}, size: {regi_size}, fixed_address: {fixed_addr}'
        else:
            ret = f'bus: {regi["bus"]}, dev: {regi["dev"]}, func: {regi["fun"]}, mask: {regi["mask"]}, width: {regi["width"]}, size: {regi_size}, fixed_address: {fixed_addr}'
        return ret

    def io_details(self, regi: Dict[str, Any]) -> str:
        regi_size = regi['size'] if 'size' in regi.keys() else None
        fixed_addr = regi['fixed_address'] if 'fixed_address' in regi.keys() else None
        if 'register' in regi.keys():
            ret = f'register: {regi["register"]}, base_field: {regi["base_field"]}, size: {regi_size}, fixed_address: {fixed_addr}'
        else:
            ret = f'bus: {regi["bus"]}, dev: {regi["dev"]}, func: {regi["fun"]}, reg: {regi["reg"]}, mask: {regi["mask"]}, size: {regi_size}, fixed_address: {fixed_addr}'
        return ret

    def mem_details(self, regi: Dict[str, Any]) -> str:
        ret = f'access: {regi["access"]}, address: {regi["address"]}, size: {regi["size"]}'
        return ret

    def control_details(self, regi: Dict[str, Any]) -> str:
        ret = f'register: {regi["register"]}, field: {regi["field"]}'
        return ret

    def lock_details(self, regi: Dict[str, Any]) -> str:
        ret = f'register: {regi["register"]}, field: {regi["field"]}, value: {regi["value"]}'
        return ret

    def bus_details(self, regi: str) -> str:
        ret = f'bus: {regi}'
        return ret


commands = {'config': CONFIGCommand}



# CHIPSEC: Platform Security Assessment Framework
# Copyright (c) 2010-2020, Intel Corporation
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; Version 2.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
#
# Contact information:
# chipsec@intel.com
#


"""
Checks for exposure of pre-boot passwords (BIOS/HDD/pre-bot authentication SW) in the BIOS keyboard buffer.

Reference:
    - DEFCON 16: `Bypassing Pre-boot Authentication Passwords by Instrumenting the BIOS Keyboard Buffer <https://www.defcon.org/images/defcon-16/dc16-presentations/brossard/defcon-16-brossard-wp.pdf>`_ by Jonathan Brossard

Usage:
    ``chipsec_main -m common.bios_kbrd_buffer``

Examples:
    >>> chipsec_main.py -m common.bios_kbrd_buffer

"""

from chipsec.module_common import BaseModule, BIOS
from chipsec.library.returncode import ModuleResult
from chipsec.library.logger import print_buffer_bytes
from typing import List

TAGS = [BIOS]
METADATA_TAGS = ['OPENSOURCE', 'IA', 'COMMON', 'BIOS_KBRD_BUFFER']

COMMON_FILL_PTRN = ''.join([f'{(chr(x + 0x1E)):1}' for x in range(32)])


class bios_kbrd_buffer(BaseModule):
    def __init__(self):
        BaseModule.__init__(self)

    def is_supported(self) -> bool:
        return True

    def check_BIOS_keyboard_buffer(self) -> int:
        kbrd_buf_head = self.cs.mem.read_physical_mem_dword(0x41A) & 0x000000FF
        kbrd_buf_tail = self.cs.mem.read_physical_mem_dword(0x41C) & 0x000000FF
        self.logger.log(f'[*] Keyboard buffer head pointer = 0x{kbrd_buf_head:X} (at 0x41A), tail pointer = 0x{kbrd_buf_tail:X} (at 0x41C)')
        bios_kbrd_buf = self.cs.mem.read_physical_mem(0x41E, 32)
        self.logger.log('[*] Keyboard buffer contents (at 0x41E):')
        print_buffer_bytes(bios_kbrd_buf)
        bios_kbrd_buf = bios_kbrd_buf.decode('latin_1')

        has_contents = False

        if COMMON_FILL_PTRN == bios_kbrd_buf:
            self.logger.log_good('Keyboard buffer is filled with common fill pattern')
            self.result.setStatusBit(self.result.status.SUCCESS)
            return self.result.getReturnCode(ModuleResult.PASSED)

        for x in bios_kbrd_buf:
            if ('\x00' != x) and ('\x20' != x):
                has_contents = True
                break

        if (0x1E < kbrd_buf_tail) and (kbrd_buf_tail <= 0x1E + 32):
            self.logger.log_bad(f'Keyboard buffer tail points inside the buffer (= 0x{kbrd_buf_tail:X})')
            self.logger.log(f'    It may potentially expose lengths of pre-boot passwords. Was your password {(kbrd_buf_tail + 2 - 0x1E) // 2:d} characters long?')

        self.logger.log('[*] Checking contents of the keyboard buffer...\n')

        if has_contents:
            self.logger.log_warning('Keyboard buffer is not empty. The test cannot determine conclusively if it contains pre-boot passwords.')
            self.logger.log('  - The contents might have not been cleared by pre-boot firmware or overwritten with garbage.')
            self.logger.log('  - Visually inspect the contents of keyboard buffer for pre-boot passwords (BIOS, HDD, full-disk encryption).')
        else:
            self.logger.log_passed('Keyboard buffer looks empty. Pre-boot passwords do not seem to be exposed.')

        if has_contents:
            self.result.setStatusBit(self.result.status.POTENTIALLY_VULNERABLE)
            return self.result.getReturnCode(ModuleResult.WARNING)
        else:
            self.result.setStatusBit(self.result.status.SUCCESS)
            return self.result.getReturnCode(ModuleResult.PASSED)

    def run(self, module_argv: List[str]) -> int:
        self.logger.start_test('Pre-boot Passwords in the BIOS Keyboard Buffer')
        self.res = self.check_BIOS_keyboard_buffer()
        return self.res



# !/usr/bin/python
# CHIPSEC: Platform Security Assessment Framework
# Copyright (c) 2010-2021, Intel Corporation
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; Version 2.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

# Contact information:
# chipsec@intel.com

"""
>>> chipsec_util cmos dump
>>> chipsec_util cmos readl|writel|readh|writeh <byte_offset> [byte_val]

Examples:

>>> chipsec_util cmos dump
>>> chipsec_util cmos readl 0x0
>>> chipsec_util cmos writeh 0x0 0xCC
"""

from argparse import ArgumentParser

from chipsec.command import BaseCommand, toLoad
from chipsec.hal.cmos import CMOS


class CMOSCommand(BaseCommand):

    def requirements(self) -> toLoad:
        return toLoad.Driver

    def parse_arguments(self) -> None:
        parser = ArgumentParser(usage=__doc__)

        parser_offset = ArgumentParser(add_help=False)
        parser_offset.add_argument('offset', type=lambda x: int(x, 0), help="offsets read")

        parser_val = ArgumentParser(add_help=False)
        parser_val.add_argument('value', type=lambda x: int(x, 0), help="value written")

        subparsers = parser.add_subparsers()

        # dump
        parser_dump = subparsers.add_parser('dump')
        parser_dump.set_defaults(func=self.cmos_dump)
        # readl
        parser_readl = subparsers.add_parser('readl', parents=[parser_offset])
        parser_readl.set_defaults(func=self.cmos_readl)
        # writel
        parser_writel = subparsers.add_parser('writel', parents=[parser_offset, parser_val])
        parser_writel.set_defaults(func=self.cmos_writel)
        # readh
        parser_readh = subparsers.add_parser('readh', parents=[parser_offset])
        parser_readh.set_defaults(func=self.cmos_readh)
        # writeh
        parser_writeh = subparsers.add_parser('writeh', parents=[parser_offset, parser_val])
        parser_writeh.set_defaults(func=self.cmos_writeh)

        parser.parse_args(self.argv, namespace=CMOSCommand)

    def set_up(self) -> None:
        self._cmos = CMOS(self.cs)

    def cmos_dump(self) -> None:
        self.logger.log("[CHIPSEC] Dumping CMOS memory..")
        self._cmos.dump()

    def cmos_readl(self) -> None:
        val = self._cmos.read_cmos_low(self.offset)
        self.logger.log(f'[CHIPSEC] CMOS low byte 0x{self.offset:X} = 0x{val:X}')

    def cmos_writel(self) -> None:
        val = self._cmos.write_cmos_low(self.offset, self.value)
        self.logger.log(f'[CHIPSEC] CMOS low byte 0x{self.offset:X} = 0x{self.value:X}')

    def cmos_readh(self) -> None:
        val = self._cmos.read_cmos_high(self.offset)
        self.logger.log(f'[CHIPSEC] CMOS high byte 0x{self.offset:X} = 0x{val:X}')

    def cmos_writeh(self) -> None:
        self.logger.log(f'[CHIPSEC] Writing CMOS high byte 0x{self.offset:X} <- 0x{self.value:X}')
        self._cmos.write_cmos_high(self.offset, self.value)


commands = {'cmos': CMOSCommand}



# CHIPSEC: Platform Security Assessment Framework
# Copyright (c) 2023, Intel Corporation
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; Version 2.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
#
# Contact information:
# chipsec@intel.com
#

BOUNDARY_1KB = 0x400
BOUNDARY_2KB = 0x800
BOUNDARY_4KB = 0x1000
BOUNDARY_1MB = 0x100000
BOUNDARY_2MB = 0x200000
BOUNDARY_4MB = 0x400000
BOUNDARY_8MB = 0x800000
BOUNDARY_16MB = 0x1000000
BOUNDARY_32MB = 0x2000000
BOUNDARY_64MB = 0x4000000
BOUNDARY_128MB = 0x8000000
BOUNDARY_256MB = 0x10000000
BOUNDARY_512MB = 0x20000000
BOUNDARY_1GB = 0x40000000
BOUNDARY_2GB = 0x80000000
BOUNDARY_4GB = 0x100000000

ALIGNED_4KB = 0xFFF
ALIGNED_1MB = 0xFFFFF
ALIGNED_8MB = 0x7FFFFF
ALIGNED_64MB = 0x3FFFFFF
ALIGNED_128MB = 0x7FFFFFF
ALIGNED_256MB = 0xFFFFFFF

SIZE_64B = 0x40
SIZE_4KB = 0x1000
SIZE_8KB = 0x2000
SIZE_16KB = 0x4000
SIZE_32KB = 0x8000
SIZE_64KB = 0x10000
SIZE_256KB = 0x40000
SIZE_512KB = 0x80000
SIZE_1MB = 0x100000
SIZE_2MB = 0x200000
SIZE_3MB = 0x300000
SIZE_4MB = 0x400000
SIZE_8MB = 0x800000
SIZE_12MB = 0xC00000
SIZE_16MB = 0x1000000
SIZE_20MB = 0x1400000
SIZE_24MB = 0x1800000
SIZE_28MB = 0x1C00000
SIZE_32MB = 0x2000000
SIZE_36MB = 0x2400000
SIZE_40MB = 0x2800000
SIZE_44MB = 0x2C00000
SIZE_48MB = 0x3000000
SIZE_52MB = 0x3400000
SIZE_56MB = 0x3800000
SIZE_60MB = 0x3C00000
SIZE_64MB = 0x4000000
SIZE_96MB = 0x6000000
SIZE_128MB = 0x8000000
SIZE_160MB = 0xA000000
SIZE_192MB = 0xC000000
SIZE_224MB = 0xE000000
SIZE_256MB = 0x10000000
SIZE_288MB = 0x12000000
SIZE_320MB = 0x14000000
SIZE_352MB = 0x16000000
SIZE_384MB = 0x18000000
SIZE_416MB = 0x1A000000
SIZE_448MB = 0x1C000000
SIZE_480MB = 0x1E000000
SIZE_512MB = 0x20000000
SIZE_1GB = 0x40000000
SIZE_1_5GB = 0x60000000
SIZE_2GB = 0x80000000
SIZE_4GB = 0x100000000
SIZE_64TB = 0x400000000000

MASK_8b = 0xFF
MASK_16b = 0xFFFF
MASK_32b = 0xFFFFFFFF
MASK_64b = 0xFFFFFFFFFFFFFFFF



"""
This module checks current contents of UEFI firmware ROM or specified firmware image for bad EFI binaries as per the
VirusTotal API. These can be EFI firmware volumes, EFI executable binaries (PEI modules, DXE drivers..) or EFI sections.
The module can find EFI binaries by their UI names, EFI GUIDs, MD5/SHA-1/SHA-256 hashes
or contents matching specified regular expressions.

Important! This module can only detect bad or vulnerable EFI modules based on the file's reputation on VT.

Usage:
    chipsec_main.py -i -m tools.uefi.reputation -a <vt_api_key>[,<vt_threshold>,<fw_image>]
      vt_api_key   : API key to VirusTotal. Can be obtained by visting https://www.virustotal.com/gui/join-us.
                     This argument must be specified.
      vt_threshold : The minimal number of different AV vendors on VT which must claim an EFI module is malicious
                     before failing the test. Defaults to 10.
      fw_image     : Full file path to UEFI firmware image
                     If not specified, the module will dump firmware image directly from ROM

.. note::
    - Requires virustotal-api

"""
import time

from chipsec.module_common import BaseModule, BIOS
from chipsec.library.returncode import ModuleResult
from chipsec.hal.spi_uefi import search_efi_tree, build_efi_model, EFIModuleType
from chipsec.hal.uefi import UEFI
from chipsec.hal.spi import SPI, BIOS
from chipsec.library.file import read_file

try:
    from virus_total_apis import PublicApi as VirusTotalPublicApi
    has_virus_total_apis = True
except ImportError:
    has_virus_total_apis = False

TAGS = [BIOS]
METADATA_TAGS = ['OPENSOURCE', 'IA', 'TOOLS', 'UEFI', 'REPUTATION']

DEF_FWIMAGE_FILE = 'fw.bin'

USAGE_TEXT = '''
Usage:

    chipsec_main.py -i -m tools.uefi.reputation -a <vt_api_key>[,<vt_threshold>,<fw_image>]

      vt_api_key   : API key to VirusTotal. Can be obtained by visiting https://www.virustotal.com/gui/join-us.
                     This argument must be specified.
      vt_threshold : The minimal number of different AV vendors on VT which must claim an EFI module is bad
                     before failing the test. Defaults to 10.
      fw_image     : Full file path to UEFI firmware image
                     If not specified, the module will dump firmware image directly from ROM
'''


class reputation(BaseModule):

    def __init__(self):
        BaseModule.__init__(self)
        self.uefi = UEFI(self.cs)
        self.image = None
        self.vt_threshold = 10
        self.vt = None

    def is_supported(self):
        if has_virus_total_apis:
            return True
        else:
            self.logger.log_important("""Can't import module 'virus_total_apis'.
Please run 'pip install virustotal-api' and try again.""")
            return False

    def reputation_callback(self, efi_module):
        vt_report = self.vt.get_file_report(efi_module.SHA256)

        while vt_report["response_code"] == 204:
            # The Public API is limited to 4 requests per minute.
            if self.logger.DEBUG:
                self.logger.log("VT API quota exceeded, sleeping for 1 minute... (-.-)zzZZ")

            time.sleep(60)
            # Retry.
            vt_report = self.vt.get_file_report(efi_module.SHA256)

        if vt_report["results"]["response_code"] == 0:
            # Hash is unknown to VT.
            self.logger.log_important(f'Unfamiliar EFI binary found in the UEFI firmware image\n{efi_module}')
            return False

        if vt_report["results"]["positives"] >= self.vt_threshold:
            self.logger.log_bad(f'Suspicious EFI binary found in the UEFI firmware image\n{efi_module}')
            return True

        if self.logger.VERBOSE:
            self.logger.log(f'Benign EFI binary found in the UEFI firmware image\n{efi_module}')

        return False

    def check_reputation(self):
        res = ModuleResult.PASSED

        # parse the UEFI firmware image and look for EFI modules matching the blocked-list
        efi_tree = build_efi_model(self.image, None)
        match_types = EFIModuleType.SECTION_EXE
        matching_modules = search_efi_tree(efi_tree, self.reputation_callback, match_types)
        found = len(matching_modules) > 0
        self.logger.log('')
        if found:
            res = ModuleResult.WARNING
            self.logger.log_warning("Suspicious EFI binary found in the UEFI firmware image")
            self.result.setStatusBit(self.result.status.POTENTIALLY_VULNERABLE)
        else:
            self.logger.log_passed("Didn't find any suspicious EFI binary")
        return res

    def usage(self):
        self.logger.log(USAGE_TEXT)

    def run(self, module_argv):
        self.logger.start_test("Check for suspicious EFI binaries in UEFI firmware")

        self.usage()

        if len(module_argv) > 0:
            self.vt = VirusTotalPublicApi(module_argv[0])
        if len(module_argv) > 1:
            self.vt_threshold = int(module_argv[1])

        image_file = DEF_FWIMAGE_FILE
        if len(module_argv) > 2:
            # Use provided firmware image
            image_file = module_argv[2]
            self.logger.log(f'[*] Reading FW image from file: {image_file}')
        else:
            # Read firmware image directly from SPI flash memory
            self.spi = SPI(self.cs)
            (base, limit, _) = self.spi.get_SPI_region(BIOS)
            image_size = limit + 1 - base
            self.logger.log(f'[*] Dumping FW image from ROM to {image_file}: 0x{base:08X} bytes at [0x{limit:08X}:0x{image_size:08X}]')
            self.logger.log("[*] This may take a few minutes (instead, use 'chipsec_util spi dump')...")
            self.spi.read_spi_to_file(base, image_size, image_file)

        self.image = read_file(image_file)

        self.res = self.check_reputation()
        return self.result.getReturnCode(self.res)



# CHIPSEC: Platform Security Assessment Framework
# Copyright (c) 2023, Intel Corporation
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; Version 2.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
#
# Contact information:
# chipsec@intel.com
#

import unittest
from unittest.mock import Mock
from chipsec.modules.common.cpu.cpu_info import cpu_info


class TestCpuInfo(unittest.TestCase):
    def test_cpu_info_init(self):
        cpu_info_instance = cpu_info()
        self.assertIsInstance(cpu_info_instance, cpu_info)

    def test_cpu_info_is_supported_false(self):
        mock_self = Mock()
        mock_self.cs.register.has_field.return_value = False
        result = cpu_info.is_supported(mock_self)
        self.assertFalse(result)

    def test_cpu_info_is_supported_true(self):
        mock_self = Mock()
        mock_self.cs.register.has_field.return_value = True
        result = cpu_info.is_supported(mock_self)
        self.assertTrue(result)


if __name__ == '__main__':
    unittest.main()



# CHIPSEC: Platform Security Assessment Framework
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; Version 2.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
#
#
import os
import platform
import tempfile
import unittest
try:
    from distro import codename, version, name
    mSystem = (name(), version(), codename())
except Exception:
    if hasattr(platform, 'linux_distribution'):
        mSystem = platform.linux_distribution()
    else:
        # Windows does not have a "distribution"
        mSystem = None


import chipsec_main


class GenericHardwareTest(unittest.TestCase):

    BOOT_MODE_LEGACY = 1
    BOOT_MODE_UEFI = 2

    def setUp(self):
        if hasattr(self, "SYSTEM") and platform.system() != self.SYSTEM:
            self.skipTest("Unsupported system {}".format(self.SYSTEM))

        if hasattr(self, "DIST") and mSystem != self.DIST:
            self.skipTest("Unsupported distribution {}".format(self.DIST))

        if (hasattr(self, "PRODUCT_NAME") and
                self.product_name() != self.PRODUCT_NAME):
            self.skipTest("Unsupported platform {}".format(self.PRODUCT_NAME))

        if (hasattr(self, "BIOS_VERSION") and
                self.bios_version() != self.BIOS_VERSION):
            self.skipTest("Unsupported BIOS version "
                          "{}".format(self.BIOS_VERSION))

        if hasattr(self, "BOOT_MODE") and self.boot_mode() != self.BOOT_MODE:
            self.skipTest("Unsupported boot type {}".format(self.BOOT_MODE))
        _, self.log_file = tempfile.mkstemp()

    def tearDown(self):
        os.remove(self.log_file)

    def _generic_main(self):
        arg = ['-l', self.log_file]
        par = chipsec_main.parse_args(arg)
        cm = chipsec_main.ChipsecMain(par, arg)
        error_code = cm.main()
        cm.logger.close()
        self.log = open(self.log_file).read()
        self.assertLessEqual(error_code, 31,
                             "At least one test raised an error")
        for test in self.PASS:
            self.assertIn("PASSED: {}".format(test), self.log)
        for test in self.WARNING:
            self.assertIn("WARNING: {}".format(test), self.log)



# CHIPSEC: Platform Security Assessment Framework
# Copyright (c) 2023, Intel Corporation
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; Version 2.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
#
# Contact information:
# chipsec@intel.com
#

"""
Banner functions
"""

import platform
import sys
from typing import Tuple, Sequence

from chipsec.chipset import Chipset
from chipsec.library.gil import gil_status
from chipsec.library.logger import logger


def chipsec_banner(arguments: Sequence[str], version: str, message: str, custom_str: str = '') -> str:
    """Creates the CHIPSEC banner string"""
    args = ' '.join(arguments)
    if custom_str:
        message += f'\n{custom_str}'
    banner = f'''
################################################################
##                                                            ##
##  CHIPSEC: Platform Hardware Security Assessment Framework  ##
##                                                            ##
################################################################
[CHIPSEC] Version  : {version}
[CHIPSEC] Arguments: {args}
{message}'''
    return banner


def print_banner(arguments: Sequence[str], version: str, message: str, custom_str: str = '') -> None:
    logger().log(chipsec_banner(arguments, version, message, custom_str))


def chipsec_banner_properties(cs: Chipset, os_version: Tuple[str, str, str, str]) -> str:
    """Creates the CHIPSEC properties banner string"""
    (system, release, version, machine) = os_version
    is_python_64 = True if (sys.maxsize > 2**32) else False
    python_version = platform.python_version()
    python_arch = '64-bit' if is_python_64 else '32-bit'
    (helper_name, driver_path) = cs.helper.get_info()
    include_pch_str = cs.Cfg.is_pch_req() or (cs.Cfg.is_pch_req() is None)

    banner_prop = f'''
[CHIPSEC] OS      : {system} {release} {version} {machine}
[CHIPSEC] Python  : {python_version} ({python_arch}) - {gil_status()} GIL
[CHIPSEC] Helper  : {helper_name} {driver_path}
[CHIPSEC] Platform: {cs.Cfg.longname}
[CHIPSEC]    CPUID: {cs.Cfg.cpuid:05X}
[CHIPSEC]      VID: {cs.Cfg.vid:04X}
[CHIPSEC]      DID: {cs.Cfg.did:04X}
[CHIPSEC]      RID: {cs.Cfg.rid:02X}'''
    if include_pch_str:
        banner_prop += f'''
[CHIPSEC] PCH     : {cs.Cfg.pch_longname}
[CHIPSEC]      VID: {cs.Cfg.pch_vid:04X}
[CHIPSEC]      DID: {cs.Cfg.pch_did:04X}
[CHIPSEC]      RID: {cs.Cfg.pch_rid:02X}
'''
    if not is_python_64 and machine.endswith('64'):
        banner_prop += 'Python architecture (32-bit) is different from OS architecture (64-bit)'

    return banner_prop


def print_banner_properties(cs: Chipset, os_version: Tuple[str, str, str, str]) -> None:

    if not cs.load_config:
        logger().log_warning('Not loading configurations. Platform will remain unknown.')
    logger().log(chipsec_banner_properties(cs, os_version))



# CHIPSEC: Platform Security Assessment Framework
# Copyright (c) 2017-2021, Intel Security
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; Version 2.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# Authors:
#   Yuriy Bulygin
#   Alex Bazhaniuk
#

"""
The module can generate a list of EFI executables from (U)EFI firmware file or
extracted from flash ROM, and then later check firmware image in flash ROM or
file against this list of expected executables

Usage:
  ``chipsec_main -m tools.uefi.scan_image [-a generate|check,<json>,<fw_image>]``
    - ``generate``  Generates a list of EFI executable binaries from the UEFI
                        firmware image (default)
    - ``check``     Decodes UEFI firmware image and checks all EFI executable
                        binaries against a specified list
    - ``json``      JSON file with configuration of allowed list EFI
                        executables (default = ``efilist.json``)
    - ``fw_image``  Full file path to UEFI firmware image. If not specified,
                        the module will dump firmware image directly from ROM

Examples:

>>> chipsec_main -m tools.uefi.scan_image

Creates a list of EFI executable binaries in ``efilist.json`` from the firmware
image extracted from ROM

>>> chipsec_main -i -n -m tools.uefi.scan_image -a generate,efilist.json,uefi.rom

Creates a list of EFI executable binaries in ``efilist.json`` from ``uefi.rom``
firmware binary

>>> chipsec_main -i -n -m tools.uefi.scan_image -a check,efilist.json,uefi.rom

Decodes ``uefi.rom`` UEFI firmware image binary and checks all EFI executables
in it against a list defined in ``efilist.json``

.. note::
    - ``-i`` and ``-n`` arguments can be used when specifying firmware file
      because the module doesn't depend on the platform and doesn't need kernel driver
"""

import json
import os

from chipsec.module_common import BaseModule, BIOS
from chipsec.library.returncode import ModuleResult

from chipsec.hal.uefi import UEFI
from chipsec.hal.spi import SPI, BIOS
from chipsec.hal.uefi_fv import EFI_MODULE, EFI_SECTION
from chipsec.hal.spi_uefi import build_efi_model, search_efi_tree, EFIModuleType, UUIDEncoder
from chipsec.library.file import write_file, read_file

TAGS = [BIOS]
METADATA_TAGS = ['OPENSOURCE', 'IA', 'TOOLS', 'UEFI', 'SCAN_IMAGE']

DEF_FWIMAGE_FILE = 'fw.bin'
DEF_EFILIST_FILE = 'efilist.json'


class scan_image(BaseModule):

    def __init__(self):
        BaseModule.__init__(self)
        self.uefi = UEFI(self.cs)
        self.image = None
        self.efi_list = {}
        self.suspect_modules = {}
        self.duplicate_list = []

    def is_supported(self):
        return True

    #
    # callbacks to uefi_search.check_match_criteria
    #
    def genlist_callback(self, efi_module: EFI_MODULE) -> None:
        md = {}
        if type(efi_module) == EFI_SECTION:
            if efi_module.SHA1:
                md["sha1"] = efi_module.SHA1
            if efi_module.parentGuid:
                md["guid"] = efi_module.parentGuid
            if efi_module.ui_string:
                md["name"] = efi_module.ui_string
            if efi_module.Name:
                md["type"] = efi_module.Name
            if efi_module.SHA256 in self.efi_list.keys():
                self.duplicate_list.append(efi_module.SHA256)
            else:
                self.efi_list[efi_module.SHA256] = md
        else:
            pass

    #
    # Generates new list of EFI executable binaries
    #
    def generate_efilist(self, json_pth: str) -> int:
        self.logger.log("[*] Generating a list of EFI executables from firmware image...")
        efi_tree = build_efi_model(self.image, None)
        search_efi_tree(efi_tree, self.genlist_callback, EFIModuleType.SECTION_EXE, True)
        self.logger.log(f'[*] Found {len(self.efi_list):d} EFI executables in UEFI firmware image \'{self.image_file}\'')
        self.logger.log(f'[*] Found {len(self.duplicate_list)} duplicate executables')
        self.logger.log_verbose(f'\t{chr(10).join(i for i in self.duplicate_list)}')
        self.logger.log(f'[*] Creating JSON file \'{json_pth}\'...')
        write_file(f'{json_pth}', json.dumps(self.efi_list, indent=2, separators=(',', ': '), cls=UUIDEncoder))
        return ModuleResult.PASSED

    #
    # Checks EFI executable binaries against allowed list
    #
    def check_list(self, json_pth: str) -> int:
        with open(json_pth) as data_file:
            self.efilist = json.load(data_file)

        self.logger.log(f'[*] Checking EFI executables against the list \'{json_pth}\'')

        # parse the UEFI firmware image and look for EFI modules matching list
        # - match only executable EFI sections (PE/COFF, TE)
        # - find all occurrences of matching EFI modules
        efi_tree = build_efi_model(self.image, None)
        search_efi_tree(efi_tree, self.genlist_callback, EFIModuleType.SECTION_EXE, True)
        self.logger.log(f'[*] Found {len(self.efi_list):d} EFI executables in UEFI firmware image \'{self.image_file}\'')

        for m in self.efi_list:
            if not (m in self.efilist):
                self.suspect_modules[m] = self.efi_list[m]
                guid = self.efi_list[m]["guid"] if 'guid' in self.efi_list[m] else '?'
                name = self.efi_list[m]["name"] if 'name' in self.efi_list[m] else '<unknown>'
                sha1 = self.efi_list[m]["sha1"] if 'sha1' in self.efi_list[m] else ''
                self.logger.log_important(f'Found EFI executable not in the list:\n    {m} (sha256)\n    {sha1} (sha1)\n    {{{guid}}}\n    {name}')

        if len(self.suspect_modules) > 0:
            self.logger.log_warning(f'Found {len(self.suspect_modules):d} EFI executables not in the list \'{json_pth}\'')
            return ModuleResult.WARNING
        else:
            self.logger.log_passed(f'All EFI executables match the list \'{json_pth}\'')
            return ModuleResult.PASSED

    def usage(self):
        self.logger.log(__doc__.replace('`', ''))

    def run(self, module_argv):
        self.logger.start_test("Simple list generation/checking for (U)EFI firmware")

        self.res = ModuleResult.NOTAPPLICABLE

        op = module_argv[0] if len(module_argv) > 0 else 'generate'

        if op in ['generate', 'check']:

            if len(module_argv) <= 2 and len(module_argv) != 0:
                self.usage()
                return self.res
            elif len(module_argv) > 2:
                json_file = module_argv[1]
                image_file = module_argv[2]
                self.logger.log(f'[*] Reading firmware from \'{image_file}\'...')
            else:
                image_file = DEF_FWIMAGE_FILE
                json_file = DEF_EFILIST_FILE
                self.spi = SPI(self.cs)
                (base, limit, _) = self.spi.get_SPI_region(BIOS)
                image_size = limit + 1 - base
                self.logger.log(f'[*] Dumping firmware image from ROM to \'{image_file}\': 0x{image_size:08X} bytes at [0x{base:08X}:0x{limit:08X}]')
                self.spi.read_spi_to_file(base, image_size, image_file)

            self.image_file = image_file
            self.image = read_file(image_file)
            json_pth = os.path.abspath(json_file)

            if op == 'generate':
                self.res = self.generate_efilist(json_pth)
            elif op == 'check':
                if not os.path.exists(json_pth):
                    self.logger.log_error(f'JSON file \'{json_file}\' doesn\'t exist. Exiting...')
                    self.res = ModuleResult.ERROR
                else:
                    self.res = self.check_list(json_pth)

        elif op == 'help':
            self.usage()
        else:
            self.logger.log_error("Unrecognized command-line argument to the module")
            self.usage()

        return self.res



# CHIPSEC: Platform Security Assessment Framework
# Copyright (c) 2018, Eclypsium, Inc.
# Copyright (c) 2019-2021, Intel Corporation
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; Version 2.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#

"""
The module checks if system includes hardware mitigations for Speculative Execution Side Channel.
Specifically, it verifies that the system supports CPU mitigations for
Branch Target Injection vulnerability a.k.a. Spectre Variant 2 (CVE-2017-5715)

The module checks if the following hardware mitigations are supported by the CPU
and enabled by the OS/software:

1. Indirect Branch Restricted Speculation (IBRS) and Indirect Branch Predictor Barrier (IBPB):
   CPUID.(EAX=7H,ECX=0):EDX[26] == 1

2. Single Thread Indirect Branch Predictors (STIBP):
   CPUID.(EAX=7H,ECX=0):EDX[27] == 1
   IA32_SPEC_CTRL[STIBP] == 1

3. Enhanced IBRS:
   CPUID.(EAX=7H,ECX=0):EDX[29] == 1
   IA32_ARCH_CAPABILITIES[IBRS_ALL] == 1
   IA32_SPEC_CTRL[IBRS] == 1

4. @TODO: Mitigation for Rogue Data Cache Load (RDCL):
   CPUID.(EAX=7H,ECX=0):EDX[29] == 1
   IA32_ARCH_CAPABILITIES[RDCL_NO] == 1

In addition to checking if CPU supports and OS enables all mitigations, we need to check
that relevant MSR bits are set consistently on all logical processors (CPU threads).


The module returns the following results:

FAILED:
    IBRS/IBPB is not supported

WARNING:
    IBRS/IBPB is supported

    Enhanced IBRS is not supported

WARNING:
    IBRS/IBPB is supported

    Enhanced IBRS is supported

    Enhanced IBRS is not enabled by the OS

WARNING:
    IBRS/IBPB is supported

    STIBP is not supported or not enabled by the OS

PASSED:
    IBRS/IBPB is supported

    Enhanced IBRS is supported

    Enhanced IBRS is enabled by the OS

    STIBP is supported


Notes:

- The module returns WARNING when CPU doesn't support enhanced IBRS
  Even though OS/software may use basic IBRS by setting IA32_SPEC_CTRL[IBRS] when necessary,
  we have no way to verify this

- The module returns WARNING when CPU supports enhanced IBRS but OS doesn't set IA32_SPEC_CTRL[IBRS]
  Under enhanced IBRS, OS can set IA32_SPEC_CTRL[IBRS] once to take advantage of IBRS protection

- The module returns WARNING when CPU doesn't support STIBP or OS doesn't enable it
  Per Speculative Execution Side Channel Mitigations:
  "enabling IBRS prevents software operating on one logical processor from controlling
  the predicted targets of indirect branches executed on another logical processor.
  For that reason, it is not necessary to enable STIBP when IBRS is enabled"

- OS/software may implement "retpoline" mitigation for Spectre variant 2
  instead of using CPU hardware IBRS/IBPB

@TODO: we should verify CPUID.07H:EDX on all logical CPUs as well
because it may differ if ucode update wasn't loaded on all CPU cores


Hardware registers used:

- CPUID.(EAX=7H,ECX=0):EDX[26]     - enumerates support for IBRS and IBPB
- CPUID.(EAX=7H,ECX=0):EDX[27]     - enumerates support for STIBP
- CPUID.(EAX=7H,ECX=0):EDX[29]     - enumerates support for the IA32_ARCH_CAPABILITIES MSR
- IA32_ARCH_CAPABILITIES[IBRS_ALL] - enumerates support for enhanced IBRS
- IA32_ARCH_CAPABILITIES[RCDL_NO]  - enumerates support RCDL mitigation
- IA32_SPEC_CTRL[IBRS]             - enable control for enhanced IBRS by the software/OS
- IA32_SPEC_CTRL[STIBP]            - enable control for STIBP by the software/OS


References:

- Reading privileged memory with a side-channel by Jann Horn, Google Project Zero:
  https://googleprojectzero.blogspot.com/2018/01/reading-privileged-memory-with-side.html

- Spectre:
  https://spectreattack.com/spectre.pdf

- Meltdown:
  https://meltdownattack.com/meltdown.pdf

- Speculative Execution Side Channel Mitigations:
  https://software.intel.com/sites/default/files/managed/c5/63/336996-Speculative-Execution-Side-Channel-Mitigations.pdf

- Retpoline: a software construct for preventing branch-target-injection:
  https://support.google.com/faqs/answer/7625886

"""

from chipsec.module_common import BaseModule, CPU, HWCONFIG, SMM
from chipsec.library.returncode import ModuleResult
from chipsec.library.exceptions import HWAccessViolationError, UnimplementedAPIError
from chipsec.library.defines import BIT26, BIT27, BIT29
from typing import List

TAGS = [CPU, HWCONFIG, SMM]
METADATA_TAGS = ['OPENSOURCE', 'IA', 'COMMON', 'CPU', 'SPECTRE_V2']


class spectre_v2(BaseModule):

    def __init__(self):
        BaseModule.__init__(self)
        self.result.url = 'https://chipsec.github.io/modules/chipsec.modules.common.cpu.spectre_v2.html'

    def is_supported(self) -> bool:
        if self.cs.is_intel():
            if self.cs.register.is_defined('IA32_ARCH_CAPABILITIES'):
                if self.cs.register.is_defined('IA32_SPEC_CTRL'):
                    return True

                self.logger.log_important('IA32_SPEC_CTRL register not defined for platform.  Skipping module.')
            else:
                self.logger.log_important('IA32_ARCH_CAPABILITIES register not defined for platform.  Skipping module.')
        elif self.cs.is_amd():
            if self.cs.register.is_defined('SPEC_CTRL'):
                return True
        else:
            self.logger.log_important('SPEC_CTRL register not defined for platform.  Skipping module.')
        return False

    def check_spectre_mitigations(self) -> int:
        try:
            cpu_thread_count = self.cs.msr.get_cpu_thread_count()
        except:
            cpu_thread_count = 1

        if self.cs.is_intel():
            #
            # Read CPUID Leaf 07H
            #
            (_, _, _, r_edx) = self.cs.cpu.cpuid(0x7, 0x0)
            ibrs_ibpb_supported = (r_edx & BIT26) > 0
            stibp_supported = (r_edx & BIT27) > 0
            arch_cap_supported = (r_edx & BIT29) > 0
            self.logger.log(f'[*] CPUID.7H:EDX[26] = {ibrs_ibpb_supported:d} Indirect Branch Restricted Speculation (IBRS) & Predictor Barrier (IBPB)')
            self.logger.log(f'[*] CPUID.7H:EDX[27] = {stibp_supported:d} Single Thread Indirect Branch Predictors (STIBP)')
            self.logger.log(f'[*] CPUID.7H:EDX[29] = {arch_cap_supported:d} IA32_ARCH_CAPABILITIES')

            if ibrs_ibpb_supported:
                self.logger.log_good('CPU supports IBRS and IBPB')
            else:
                self.logger.log_bad('CPU does not support IBRS and IBPB')

            if stibp_supported:
                self.logger.log_good('CPU supports STIBP')
            else:
                self.logger.log_bad('CPU does not support STIBP')

            if arch_cap_supported:
                ibrs_enh_supported = True
                self.logger.log('[*] Checking enhanced IBRS support in IA32_ARCH_CAPABILITIES...')
                for tid in range(cpu_thread_count):
                    arch_cap_msr = 0
                    try:
                        arch_cap_msr = self.cs.register.read('IA32_ARCH_CAPABILITIES', tid)
                    except HWAccessViolationError:
                        self.logger.log_error('Could not read IA32_ARCH_CAPABILITIES')
                        ibrs_enh_supported = False
                        break

                    ibrs_all = self.cs.register.get_field('IA32_ARCH_CAPABILITIES', arch_cap_msr, 'IBRS_ALL')
                    self.logger.log(f'[*]   cpu{tid:d}: IBRS_ALL = {ibrs_all:x}')
                    if 0 == ibrs_all:
                        ibrs_enh_supported = False
                        break

                if ibrs_enh_supported:
                    self.logger.log_good('CPU supports enhanced IBRS (on all logical CPU)')
                else:
                    self.logger.log_bad('CPU does not support enhanced IBRS')
            else:
                ibrs_enh_supported = False
                self.logger.log_bad('CPU does not support enhanced IBRS')

            ibrs_enabled = True
            stibp_enabled_count = 0
            if ibrs_enh_supported:
                self.logger.log('[*] Checking if OS is using Enhanced IBRS...')
                for tid in range(cpu_thread_count):
                    spec_ctrl_msr = 0
                    try:
                        spec_ctrl_msr = self.cs.register.read('IA32_SPEC_CTRL', tid)
                    except HWAccessViolationError:
                        self.logger.log_error('Could not read IA32_SPEC_CTRL')
                        ibrs_enabled = False
                        break

                    ibrs = self.cs.register.get_field('IA32_SPEC_CTRL', spec_ctrl_msr, 'IBRS')
                    self.logger.log(f'[*]   cpu{tid:d}: IA32_SPEC_CTRL[IBRS] = {ibrs:x}')
                    if 0 == ibrs:
                        ibrs_enabled = False

                    # ok to access STIBP bit even if STIBP is not supported
                    stibp = self.cs.register.get_field('IA32_SPEC_CTRL', spec_ctrl_msr, 'STIBP')
                    self.logger.log(f'[*]   cpu{tid:d}: IA32_SPEC_CTRL[STIBP] = {stibp:x}')
                    if 1 == stibp:
                        stibp_enabled_count += 1

                if ibrs_enabled:
                    self.logger.log_good('OS enabled Enhanced IBRS (on all logical processors)')
                else:
                    self.logger.log_bad('OS does not seem to use Enhanced IBRS')
                if stibp_enabled_count == cpu_thread_count:
                    self.logger.log_good('OS enabled STIBP (on all logical processors)')
                elif stibp_enabled_count > 0:
                    self.logger.log_good('OS selectively enabling STIBP')
                else:
                    self.logger.log_information('Unable to determine if the OS uses STIBP')

            #
            # Combining results of all checks into final decision
            #
            # FAILED : IBRS/IBPB is not supported
            # WARNING: IBRS/IBPB is supported
            #          enhanced IBRS is not supported
            # WARNING: IBRS/IBPB is supported
            #          enhanced IBRS is supported
            #          enhanced IBRS is not enabled by the OS
            # WARNING: IBRS/IBPB is supported
            #          STIBP is not supported
            # PASSED : IBRS/IBPB is supported
            #          enhanced IBRS is supported
            #          enhanced IBRS is enabled by the OS
            #          STIBP is supported
            #
            if not ibrs_ibpb_supported:
                res = ModuleResult.FAILED
                self.result.setStatusBit(self.result.status.MITIGATION)
                self.logger.log_failed('CPU mitigation (IBRS) is missing')
            elif not ibrs_enh_supported:
                res = ModuleResult.WARNING
                self.result.setStatusBit(self.result.status.PROTECTION)
                self.logger.log_warning('CPU supports mitigation (IBRS) but does not support enhanced IBRS')
            elif ibrs_enh_supported and (not ibrs_enabled):
                res = ModuleResult.WARNING
                self.result.setStatusBit(self.result.status.MITIGATION)
                self.logger.log_warning('CPU supports mitigation (enhanced IBRS) but OS is not using it')
            else:
                if not stibp_supported:
                    res = ModuleResult.WARNING
                    self.result.setStatusBit(self.result.status.MITIGATION)
                    self.logger.log_warning('CPU supports mitigation (enhanced IBRS) but STIBP is not supported')
                else:
                    res = ModuleResult.PASSED
                    self.logger.log_passed('CPU and OS support hardware mitigations')

            self.logger.log_important('OS may be using software based mitigation (eg. retpoline)')
            try:
                if self.cs.helper.retpoline_enabled():
                    res = ModuleResult.PASSED
                    self.logger.log_passed('Retpoline is enabled by the OS')
                else:
                    self.logger.log_bad('Retpoline is NOT enabled by the OS')
            except UnimplementedAPIError as e:
                self.logger.log_warning(str(e))
            except NotImplementedError:
                self.logger.log_warning('Retpoline check not implemented in current environment')

        elif self.cs.is_amd():
            res = ModuleResult.PASSED
            fields = ['IBRS', 'STIBP', 'SSBD', 'PSFD']
            settings = {}
            threads = {}
            fail = False

            for tid in range(cpu_thread_count):
                spec_ctrl_msr = self.cs.register.read('SPEC_CTRL', tid)
                for f in fields:
                    settings[f] = bool(self.cs.register.get_field('SPEC_CTRL', spec_ctrl_msr, f))
                if not (settings[f]):
                    fail = True
                threads[tid] = settings

            if fail:
                self.logger.log_failed("Spectre V2 Mitigations Currently not enabled")
                res = ModuleResult.PASSED
                for t, s in threads.items():
                    self.logger.log_warning(f"Thread[{t}]: {fields[0]}:{s[fields[0]]} {fields[1]}:{s[fields[1]]} {fields[2]}:{s[fields[2]]} {fields[3]}:{s[fields[3]]}")
            else:
                self.logger.log_passed("Spectre V2 Mitigations Currently enabled")
        return res

    def run(self, module_argv: List[str]) -> int:
        self.logger.start_test('Checks for Branch Target Injection / Spectre v2 (CVE-2017-5715)')
        self.res = self.check_spectre_mitigations()
        return self.result.getReturnCode(self.res)



# CHIPSEC: Platform Security Assessment Framework
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; Version 2.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
#
#
import os
import tempfile
import unittest

from tests.software import mock_helper

from chipsec.library import logger
from chipsec import chipset


class TestChipsecCs(unittest.TestCase):
    """Test the commands exposed by chipsec_utils.

    Each test may define its virtual helper and then call the _chipsec_util
    method with the command line arguments.
    """

    def setUp(self):
        """Setup the environment for the utils tests.

        We mock the helper registry to only contain our emulated helper.
        """
        fileno, self.log_file = tempfile.mkstemp()
        os.close(fileno)
        chipset._chipset = None

    def tearDown(self):
        os.remove(self.log_file)
        chipset._chipset = None

    def _chipsec_cs(self, arg, helper_class=mock_helper.TestHelper, platform=None, pch=None):
        """Run the chipsec chipset commands

        Each test may setup a virtual helper to emulate the expected behaviour
        from the hardware. If no helper is provided, TestHelper will be used.
        It verifies that no error is being reported. self.log will be populated
        with the output.
        """
        _cs = chipset.cs()
        logger.logger().HAL = True
        logger.logger().VERBOSE = True
        logger.logger().set_log_file(self.log_file)
        try:
            _cs.init(platform, pch, helper_class())
            ret = getattr(_cs.Cfg, arg.split()[0])()
        finally:
            logger.logger().close()
        with open(self.log_file, 'rb') as log:
            self.log = log.read()
        return ret


from chipsec.library.register import RegisterType
class DummyCS:
    class DummyCfg:
        REGISTERS = {
            'TEST_MSR': {
            'type': RegisterType.MSR,
            'msr': 0x123,
            'size': 4,
            'desc': "Test MSR Register",
            'FIELDS': {
                'FIELD1': {'bit': 0, 'size': 8, 'desc': "Test Field 1"},
            }
            },
            'TEST_PCI': {
            'type': RegisterType.PCICFG,
            'device': 'DEV1',
            'dev': 1,
            'fun': 0,
            'offset': 0x10,
            'size': 4,
            'desc': "Test PCI Register",
            'FIELDS': {
                'FIELD2': {'bit': 8, 'size': 8, 'desc': "Test PCI Field 2"}
            }
            },
            'TEST_MMIO': {
            'type': RegisterType.MMIO,
            'bar': 'TEST_MMIO_BAR',
            'offset': 0x20,
            'size': 4,
            'desc': "Test MMIO Register",
            'FIELDS': {
                'FIELD3': {'bit': 16, 'size': 8, 'desc': "Test MMIO Field 3"}
            }
            },
            'TEST_BAD_MMIO': {
            'type': RegisterType.MMIO,
            'bar': 'BAD',
            'offset': 0x20,
            'size': 4,
            'desc': "Test MMIO Register",
            'FIELDS': {
                'FIELD3': {'bit': 16, 'size': 8, 'desc': "Test MMIO Field 3"}
            }
            }
        }
        MMIO_BARS = {
            'TEST_MMIO_BAR': {
            'device': 'DEV1',
            'bar': 0,
            'desc': "Test MMIO BAR"
            }
        }
        CONFIG_PCI = {
            'DEV1': {'dev': 1, 'fun': 0}
        }
        MEMORY_RANGES = {}
        IMA_REGISTERS = {}

    class DummyDevice:
        def get_bus(self, device): return [0]
        def get_first_bus(self, reg): return 0
        def get_first(self, bus): return bus

    class DummyMMIO:
        def get_MMIO_BAR_base_address(self, bar, bus=None): return (0,) if bar == "BAD" else (1, )
        def read_MMIO_BAR_reg(self, bar, offset, size, bus): return 0xDEADBEEF
        def read_MMIO_reg(self, address, offset, size): return 0xBEEF
        def read_mmcfg_reg(self, b, d, f, o, size): return 0xFEED
        def write_MMIO_BAR_reg(self, bar, offset, value, size, bus): pass
        def write_MMIO_reg(self, address, offset, value, size): pass
        def write_mmcfg_reg(self, b, d, f, o, size, value): pass

    class DummyMSR:
        def read_msr(self, cpu_thread, msr): return (0xABCD, 0x1234)
        def write_msr(self, cpu_thread, msr, eax, edx): pass

    class DummyIO:
        def _read_port(self, port, size): return 0xFF
        def _write_port(self, port, value, size): pass

    class DummyIOBAR:
        def get_IO_BAR_base_address(self, bar): return (1, )
        def read_IO_BAR_reg(self, bar, offset, size): return 0xCAFE
        def write_IO_BAR_reg(self, bar, offset, size, value): pass

    class DummyMsgBus:
        def msgbus_reg_read(self, port, offset): return 0xAABB
        def mm_msgbus_reg_read(self, port, offset): return 0xCCDD
        def msgbus_reg_write(self, port, offset, value): pass
        def mm_msgbus_reg_write(self, port, offset, value): pass

    class DummyMem:
        def write_physical_mem(self, address, size, value): pass

    class DummyCPU:
        def get_cpu_topology(self):
            return {'packages': {0: [0]}, 'cores': {0: [0]}}

    class DummyHelper:
        def get_threads_count(self): return 1

    def __init__(self):
        self.Cfg = self.DummyCfg()
        self.device = self.DummyDevice()
        self.mmio = self.DummyMMIO()
        self.msr = self.DummyMSR()
        self.io = self.DummyIO()
        self.iobar = self.DummyIOBAR()
        self.msgbus = self.DummyMsgBus()
        self.mem = self.DummyMem()
        self.cpu = self.DummyCPU()
        self.helper = self.DummyHelper()
        self.consistency_checking = False


# -*- coding: utf-8 -*-
# CHIPSEC: Platform Security Assessment Framework
# Copyright (c) 2021, SentinelOne
# Copyright (c) 2021, Intel
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; Version 2.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

"""
SMM_CODE_CHK_EN (SMM Call-Out) Protection check

SMM_CODE_CHK_EN is a bit found in the MSR_SMM_FEATURE_CONTROL register.
Once set to '1', any CPU that attempts to execute SMM code not within the ranges defined by the SMRR will assert an unrecoverable MCE.
As such, enabling and locking this bit is an important step in mitigating SMM call-out vulnerabilities.
This CHIPSEC module simply reads the register and checks that SMM_CODE_CHK_EN is set and locked.

Reference:
    - Intel 64 and IA-32 Architectures Software Developer Manual (SDM)
        - https://www.intel.com/content/www/us/en/developer/articles/technical/intel-sdm.html

Usage:
    ``chipsec_main -m common.smm_code_chk``

Examples:
    >>> chipsec_main.py -m common.smm_code_chk

Registers used:
    - MSR_SMM_FEATURE_CONTROL.LOCK
    - MSR_SMM_FEATURE_CONTROL.SMM_CODE_CHK_EN

.. note::
    - MSR_SMM_FEATURE_CONTROL may not be defined or readable on all platforms.

"""
from chipsec.library.exceptions import HWAccessViolationError
from chipsec.module_common import BaseModule, BIOS, SMM
from chipsec.library.returncode import ModuleResult
from typing import List

TAGS = [BIOS, SMM]
METADATA_TAGS = ['OPENSOURCE', 'IA', 'COMMON', 'SMM_CODE_CHK']


class smm_code_chk(BaseModule):

    def __init__(self):
        BaseModule.__init__(self)

    def is_supported(self) -> bool:
        if not self.cs.register.is_defined('MSR_SMM_FEATURE_CONTROL'):
            # The MSR_SMM_FEATURE_CONTROL register is available starting from:
            # * 4th Generation Intel® Core™ Processors (Haswell microarchitecture)
            # * Atom Processors Based on the Goldmont Microarchitecture
            self.logger.log_important('Register MSR_SMM_FEATURE_CONTROL not defined for platform.  Skipping module.')
            return False

        # The Intel SDM states that MSR_SMM_FEATURE_CONTROL can only be accessed while the CPU executes in SMM.
        # However, in reality many users report that there is no problem reading this register from outside of SMM.
        # Just to be on the safe side of things, we'll verify we can read this register successfully before moving on.
        try:
            self.cs.register.read('MSR_SMM_FEATURE_CONTROL')
        except HWAccessViolationError:
            self.logger.log_important('MSR_SMM_FEATURE_CONTROL is unreadable.  Skipping module.')
            return False
        else:
            return True

    def _check_SMM_Code_Chk_En(self, thread_id: int) -> int:
        regval = self.cs.register.read('MSR_SMM_FEATURE_CONTROL', thread_id)
        lock = self.cs.register.get_field('MSR_SMM_FEATURE_CONTROL', regval, 'LOCK')
        code_chk_en = self.cs.register.get_field('MSR_SMM_FEATURE_CONTROL', regval, 'SMM_CODE_CHK_EN')

        self.cs.register.print('MSR_SMM_FEATURE_CONTROL', regval, cpu_thread=thread_id)

        if 1 == code_chk_en:
            if 1 == lock:
                res = ModuleResult.PASSED
            else:
                res = ModuleResult.FAILED
                self.result.setStatusBit(self.result.status.LOCKS)
        else:
            # MSR_SMM_MCA_CAP (the register that reports enhanced SMM capabilities) can only be read from SMM.
            # Thus, there is no way to tell whether the the CPU doesn't support SMM_CODE_CHK_EN in the first place,
            # or the CPU supports SMM_CODE_CHK_EN but the BIOS forgot to enable it.
            #
            # In either case, there is nothing that prevents SMM code from executing instructions outside the ranges defined by the SMRRs,
            # so we should at least issue a warning regarding that.
            res = ModuleResult.WARNING

        return res

    def check_SMM_Code_Chk_En(self) -> int:

        results = []
        for tid in range(self.cs.msr.get_cpu_thread_count()):
            results.append(self._check_SMM_Code_Chk_En(tid))

        if not all(_ == results[0] for _ in results):
            self.logger.log_failed('MSR_SMM_FEATURE_CONTROL does not have the same value across all CPUs')
            self.result.setStatusBit(self.result.status.POTENTIALLY_VULNERABLE)
            return ModuleResult.FAILED

        res = results[0]
        if res == ModuleResult.FAILED:
            self.logger.log_failed('SMM_Code_Chk_En is enabled but not locked down')
            self.result.setStatusBit(self.result.status.LOCKS)
        elif res == ModuleResult.WARNING:
            self.logger.log_warning("""[*] SMM_Code_Chk_En is not enabled.
This can happen either because this feature is not supported by the CPU or because the BIOS forgot to enable it.
Please consult the Intel SDM to determine whether or not your CPU supports SMM_Code_Chk_En.""")
            self.result.setStatusBit(self.result.status.VERIFY)
        else:
            self.logger.log_passed('SMM_Code_Chk_En is enabled and locked down')

        return self.result.getReturnCode(res)

    def run(self, module_argv: List[str]) -> int:
        self.logger.start_test('SMM_Code_Chk_En (SMM Call-Out) Protection')
        self.res = self.check_SMM_Code_Chk_En()
        return self.res



# -*- coding: utf-8 -*-
#
# CHIPSEC documentation build configuration file, created by
# sphinx-quickstart on Wed Mar 25 13:24:44 2015.
#
# This file is execfile()d with the current directory set to its
# containing dir.
#
# Note that not all possible configuration values are present in this
# autogenerated file.
#
# All configuration values have a default; values that are commented out
# serve to show the default.

import sys
import os
import shlex

# If extensions (or modules to document with autodoc) are in another directory,
# add these directories to sys.path here. If the directory is relative to the
# documentation root, use os.path.abspath to make it absolute, like shown here.
sys.path.insert(0, os.path.abspath(os.path.join('.', '..', '..')))

# -- General configuration ------------------------------------------------

# If your documentation needs a minimal Sphinx version, state it here.
#needs_sphinx = '1.0'

# Add any Sphinx extension module names here, as strings. They can be
# extensions coming with Sphinx (named 'sphinx.ext.*') or your custom
# ones.
extensions = [
    'sphinx.ext.autodoc',
    'rst2pdf.pdfbuilder',
    'sphinx.ext.doctest',
    'sphinx.ext.todo',
    'sphinx.ext.coverage',
    'sphinx.ext.ifconfig',
    'sphinx.ext.viewcode',
]

# Add any paths that contain templates here, relative to this directory.
# templates_path = []

# The suffix(es) of source filenames.
# You can specify multiple suffix as a list of string:
# source_suffix = ['.rst', '.md']
source_suffix = '.rst'

# The encoding of source files.
#source_encoding = 'utf-8-sig'

autodoc_mock_imports = []
mock_path = os.path.join(os.getcwd(), '_mock')
for file in os.listdir(mock_path):
    with open(os.path.join(mock_path, file), 'r') as f:
        mock_imports = f.read()
    for mock in mock_imports.split(','):
        autodoc_mock_imports.append(mock)


# The toctree document.
main_doc = 'index'

# General information about the project.
project = u'CHIPSEC'
copyright = u'(c) 2010-2021 Intel Corporation'
author = u'CHIPSEC'

# The version info for the project you're documenting, acts as replacement for
# |version| and |release|, also used in various other places throughout the
# built documents.
#
# The short X.Y version.
#version = ''
# The full version, including alpha/beta/rc tags.
#release = ''

# The language for content autogenerated by Sphinx. Refer to documentation
# for a list of supported languages.
#
# This is also used if you do content translation via gettext catalogs.
# Usually you set "language" from the command line for these cases.
language = 'en'

# There are two options for replacing |today|: either, you set today to some
# non-false value, then it is used:
#today = ''
# Else, today_fmt is used as the format for a strftime call.
#today_fmt = '%B %d, %Y'

# List of patterns, relative to source directory, that match files and
# directories to ignore when looking for source files.
exclude_patterns = ['_build']

# The reST default role (used for this markup: `text`) to use for all
# documents.
#default_role = None

# If true, '()' will be appended to :func: etc. cross-reference text.
#add_function_parentheses = True

# If true, the current module name will be prepended to all description
# unit titles (such as .. function::).
add_module_names = False

# If true, sectionauthor and moduleauthor directives will be shown in the
# output. They are ignored by default.
#show_authors = False

# The name of the Pygments (syntax highlighting) style to use.
pygments_style = 'sphinx'

# A list of ignored prefixes for module index sorting.
modindex_common_prefix = ['chipsec', 'chipsec.', 'chipsec_']

# If true, keep warnings as "system message" paragraphs in the built documents.
#keep_warnings = False

# If true, `todo` and `todoList` produce output, else they produce nothing.
todo_include_todos = False


# -- Options for HTML output ----------------------------------------------

# The theme to use for HTML and HTML Help pages.  See the documentation for
# a list of builtin themes.
html_theme = 'classic'

# Theme options are theme-specific and customize the look and feel of a theme
# further.  For a list of options available for each theme, see the
# documentation.
html_theme_options = {
    'body_min_width':'80%',
    'body_max_width':'150%',
    'sidebarbgcolor':'#41728A',
    'sidebarlinkcolor':'#B9D6E5',
    'headtextcolor':'#86B3CA',
    'headbgcolor':'#183544',
    'relbarbgcolor':'#525252',
    'relbarlinkcolor':'#E9E9E9',
    'footerbgcolor':'#262626',
    'visitedlinkcolor':'#8F5DA2',
    'linkcolor':'#548FAD'
}

# Add any paths that contain custom themes here, relative to this directory.
#html_theme_path = []

# The name for this set of Sphinx documents.  If None, it defaults to
# "<project> v<release> documentation".
# html_title = 'CHIPSEC'

# A shorter title for the navigation bar.  Default is the same as html_title.
#html_short_title = None

# The name of an image file (relative to this directory) to place at the top
# of the sidebar.
html_logo = "_images/chipsec_logo_transparent.png"

# The name of an image file (within the static path) to use as favicon of the
# docs.  This file should be a Windows icon file (.ico) being 16x16 or 32x32
# pixels large.
html_favicon = "_images/chipsec_favicon.jpg"

# Add any paths that contain custom static files (such as style sheets) here,
# relative to this directory. They are copied after the builtin static files,
# so a file named "default.css" will overwrite the builtin "default.css".
#html_static_path = []

# Add any extra paths that contain custom files (such as robots.txt or
# .htaccess) here, relative to this directory. These files are copied
# directly to the root of the documentation.
#html_extra_path = []

# If not '', a 'Last updated on:' timestamp is inserted at every page bottom,
# using the given strftime format.
html_last_updated_fmt = '%b %d, %Y'

# If true, SmartyPants will be used to convert quotes and dashes to
# typographically correct entities.
#html_use_smartypants = True

# Custom sidebar templates, maps document names to template names.
html_sidebars = {'**':['globaltoc.html', 'relations.html', 'searchbox.html']}

# Additional templates that should be rendered to pages, maps page names to
# template names.
#html_additional_pages = {}

# If false, no module index is generated.
#html_domain_indices = True

# If false, no index is generated.
#html_use_index = True

# If true, the index is split into individual pages for each letter.
#html_split_index = False

# If true, links to the reST sources are added to the pages.
html_show_sourcelink = False

# If true, "Created using Sphinx" is shown in the HTML footer. Default is True.
#html_show_sphinx = True

# If true, "(C) Copyright ..." is shown in the HTML footer. Default is True.
html_show_copyright = False

# If true, an OpenSearch description file will be output, and all pages will
# contain a <link> tag referring to it.  The value of this option must be the
# base URL from which the finished HTML is served.
#html_use_opensearch = ''

# This is the file name suffix for HTML files (e.g. ".xhtml").
#html_file_suffix = None

# Language to be used for generating the HTML full-text search index.
# Sphinx supports the following languages:
#   'da', 'de', 'en', 'es', 'fi', 'fr', 'hu', 'it', 'ja'
#   'nl', 'no', 'pt', 'ro', 'ru', 'sv', 'tr'
#html_search_language = 'en'

# A dictionary with options for the search language support, empty by default.
# Now only 'ja' uses this config value
#html_search_options = {'type': 'default'}

# The name of a javascript file (relative to the configuration directory) that
# implements a search results scorer. If empty, the default will be used.
#html_search_scorer = 'scorer.js'

# Output file base name for HTML help builder.
htmlhelp_basename = 'CHIPSECdoc'

# -- Options for LaTeX output ---------------------------------------------

latex_elements = {
# The paper size ('letterpaper' or 'a4paper').
#'papersize': 'letterpaper',

# The font size ('10pt', '11pt' or '12pt').
'pointsize': '10pt',

# Additional stuff for the LaTeX preamble.
#'preamble': '',

# Latex figure (float) alignment
#'figure_align': 'htbp',
}

# Grouping the document tree into LaTeX files. List of tuples
# (source start file, target name, title,
#  author, documentclass [howto, manual, or own class]).
latex_documents = [
  (main_doc, 'CHIPSEC.tex', u'CHIPSEC Documentation',
   u'', 'manual'),
]

# The name of an image file (relative to this directory) to place at the top of
# the title page.
#latex_logo = None

# For "manual" documents, if this is true, then toplevel headings are parts,
# not chapters.
#latex_use_parts = False

# If true, show page references after internal links.
#latex_show_pagerefs = False

# If true, show URL addresses after external links.
#latex_show_urls = False

# Documents to append as an appendix to all manuals.
#latex_appendices = []

# If false, no module index is generated.
#latex_domain_indices = True


# -- Options for manual page output ---------------------------------------

# One entry per manual page. List of tuples
# (source start file, name, description, authors, manual section).
man_pages = [
    (main_doc, 'chipsec', u'CHIPSEC Documentation',
     [author], 1)
]

# If true, show URL addresses after external links.
#man_show_urls = False


# -- Options for Texinfo output -------------------------------------------

# Grouping the document tree into Texinfo files. List of tuples
# (source start file, target name, title, author,
#  dir menu entry, description, category)
texinfo_documents = [
  (main_doc, 'CHIPSEC', u'CHIPSEC Documentation',
   author, 'CHIPSEC', 'Platform Security Assessment Framework',
   'Miscellaneous'),
]

# Documents to append as an appendix to all manuals.
#texinfo_appendices = []

# If false, no module index is generated.
#texinfo_domain_indices = True

# How to display URL addresses: 'footnote', 'no', or 'inline'.
#texinfo_show_urls = 'footnote'

# If true, do not generate a @detailmenu in the "Top" node's menu.
#texinfo_no_detailmenu = False


# -- Options for Epub output ----------------------------------------------

# Bibliographic Dublin Core info.
# epub_title = project
# epub_author = author
# epub_publisher = author
# epub_copyright = copyright

# The basename for the epub file. It defaults to the project name.
#epub_basename = project

# The HTML theme for the epub output. Since the default themes are not optimized
# for small screen space, using the same theme for HTML and epub output is
# usually not wise. This defaults to 'epub', a theme designed to save visual
# space.
#epub_theme = 'epub'

# The language of the text. It defaults to the language option
# or 'en' if the language is not set.
#epub_language = ''

# The scheme of the identifier. Typical schemes are ISBN or URL.
#epub_scheme = ''

# The unique identifier of the text. This can be a ISBN number
# or the project homepage.
#epub_identifier = ''

# A unique identification for the text.
#epub_uid = ''

# A tuple containing the cover image and cover page html template filenames.
#epub_cover = ()

# A sequence of (type, uri, title) tuples for the guide element of content.opf.
#epub_guide = ()

# HTML files that should be inserted before the pages created by sphinx.
# The format is a list of tuples containing the path and title.
#epub_pre_files = []

# HTML files shat should be inserted after the pages created by sphinx.
# The format is a list of tuples containing the path and title.
#epub_post_files = []

# A list of files that should not be packed into the epub file.
epub_exclude_files = ['search.html']

# The depth of the table of contents in toc.ncx.
#epub_tocdepth = 3

# Allow duplicate toc entries.
#epub_tocdup = True

# Choose between 'default' and 'includehidden'.
#epub_tocscope = 'default'

# Fix unsupported image types using the Pillow.
#epub_fix_images = False

# Scale large images.
#epub_max_image_width = 0

# How to display URL addresses: 'footnote', 'no', or 'inline'.
#epub_show_urls = 'inline'

# If false, no index is generated.
#epub_use_index = True

# -- Options for PDF output --------------------------------------------------


# Grouping the document tree into PDF files. List of tuples
# (source start file, target name, title, author, options).
#
# If there is more than one author, separate them with \\.
# For example: r'Guido van Rossum\\Fred L. Drake, Jr., editor'
#
# The options element is a dictionary that lets you override
# this config per-document.
# For example,
# ('index', u'MyProject', u'My Project', u'Author Name',
#  dict(pdf_compressed = True))
# would mean that specific document would be compressed
# regardless of the global pdf_compressed setting.


pdf_documents = [
('index', u'chipsec-manual', u'CHIPSEC', u'Platform Security Assessment Framework'),
]


# A comma-separated list of custom stylesheets. Example:
pdf_stylesheets = ['sstylesheet']

# A list of folders to search for stylesheets. Example:
pdf_style_path = ['./_templates']


# Create a compressed PDF
# Use True/False or 1/0
# Example: compressed=True
#pdf_compressed = False


# A colon-separated list of folders to search for fonts. Example:
pdf_font_path = [ './_templates', '/usr/share/fonts']


# Language to be used for hyphenation support
#pdf_language = "en_US"


# Mode for literal blocks wider than the frame. Can be
# overflow, shrink or truncate
pdf_fit_mode = "shrink"


# Section level that forces a break page.
# For example: 1 means top-level sections start in a new page
# 0 means disabled
#pdf_break_level = 0


# When a section starts in a new page, force it to be 'even', 'odd',
# or just use 'any'
#pdf_breakside = 'any'


# Insert footnotes where they are defined instead of
# at the end.
#pdf_inline_footnotes = True


# verbosity level. 0 1 or 2
#pdf_verbosity = 0


# If false, no index is generated.
pdf_use_index = False


# If false, no modindex is generated.
pdf_use_modindex = False


# If false, no coverpage is generated.
#pdf_use_coverpage = True


# Name of the cover page template to use
pdf_cover_template = '_templates/scover.tmpl'


# Documents to append as an appendix to all manuals.
#pdf_appendices = []


# Enable experimental feature to split table cells. Use it
# if you get "DelayedTable too big" errors
#pdf_splittables = False


# Set the default DPI for images
#pdf_default_dpi = 72


# Enable rst2pdf extension modules (default is only vectorpdf)
# you need vectorpdf if you want to use sphinx's graphviz support
#pdf_extensions = ['vectorpdf']


# Page template name for "regular" pages
pdf_page_template = 'mainPage'


# Show Table Of Contents at the beginning?
#pdf_use_toc = True


# How many levels deep should the table of contents be?
pdf_toc_depth = 99


# Add section number to section references
pdf_use_numbered_links = True


# Background images fitting mode
pdf_fit_background_mode = 'scale'
